/////////////////////////////////////////////////////////////////////////////
///
/// \file io.c
///
/// \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
///
/// \brief File that defines the IN/OUT definitions and functions 
///
/// \version 1.02
///
/// \date September 2008
///
///
/// This file will define all I/O functions used in the software
///
///
/////////////////////////////////////////////////////////////////////////////


/*Intern inclusions*/
#include "io.h"
#include "macros.h"
#include "options.h"




////////////////////////////////////////////////////////////////////////////
///
/// \fn void read_file(FILE *fd, char **content)
///
/// \brief Function used to read a file and get its content
///
/// \param[in] fd : the file descriptor of the reading file
/// \param[out] content : the string representation of the file content
///
/// This function will be used to read a file and return its content or NULL
/// if the file is empty
///
/////////////////////////////////////////////////////////////////////////////

void read_file(FILE* fd, char **content) 
{

  /*Length of the file*/
  long length;
 
  /*Go to the end of the file to know the length of the file*/
  fseek(fd,0,SEEK_END);
  length = ftell(fd);

  /*Allocate the content and read the file*/
  fseek(fd,0,SEEK_SET);

  /*If the file is not empty read it*/
  if(length != 0){
    NEW(*content, char, length);
    fread(*content, sizeof(char), length - 1, fd);
  }
  else{
    *content = NULL;
  }

   
}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void get_size_number(int num, int *size)
///
/// \brief Function used to get the number of chars taken to represent an 
/// integer into a string
///
/// \param[in] num : the number to consider
/// \param[in,out] size : the calculated size
///
/// This function will be used to get the number of chars taken to 
/// represent a integer into a string
///
/////////////////////////////////////////////////////////////////////////////

void get_size_number(int num, int *size)
{

  /*According to the number get the size*/
  if(num < 10){
    *size = 1;
  }
  else{
    if(num < 100){
      *size = 2;
    }
    else{
      if(num < 1000){
	*size = 3;
      }
      else{
	*size = 4;
      }
    }
  }

}




////////////////////////////////////////////////////////////////////////////
///
/// \fn int write_GFF3(p_matt_ygraph m)
///
/// \brief Function used to write GFF3 files
///
/// \param[in] m : the main structure containing all info needed
///
/// \return the number of files written
///
/// This function will produce as many GFF3 as the number of sets of 
/// sequences discovered
///
/////////////////////////////////////////////////////////////////////////////


int write_GFF3(p_matt_ygraph m)
{

  /*Indexes*/
  int i,j;
  /*Current filename*/
  char *filename = NULL;
  /*Size in char of the number of the filename*/
  int size;
  /*Current file descriptor*/
  FILE *fd;
  /*Temporary query index*/
  int temp_index = -1;
  /*Temporary query index in the sequence array*/
  int temp_index_seq = -1;
  /*Temporary query index in the query interval array*/
  int temp_index_clique = -1;
  /*Number of sequences by set wanted*/
  int nb_seqs_set = 0;
  /*Number of current set processed*/
  int nb_set = 0;

  /*Check the number of sequences by set wanted : if -1 all seqs wanted*/
  if(OPTS_nb_seqs_set != -1){
    nb_seqs_set = OPTS_nb_seqs_set - 1;
  }
  else{
    nb_seqs_set = 100000;
  }

  /*Get all sets*/
  for(i = 0; i < m->sets_of_sequences->nb_sets; i++){

    /*If no queries : no output*/
    if(((m->sets_of_sequences->sets[i]->nb_q_intervals != 0) && 
	OPTS_selecting_mode == 0) ||
       ((OPTS_selecting_mode != 0) && 
	(m->sets_of_sequences->sets[i]->clique_set != NULL) &&
	(m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique != 0))){

      nb_set++;

      /*Open the new file*/
      get_size_number(nb_set, &size);
      NEW(filename,
	  char,
	  strlen(OPTS_output_dir) + size + strlen(".gff") + 1);
      sprintf(filename,
	      "%s%i%s",
	      OPTS_output_dir,
	      nb_set,
	      ".gff");
      fd = fopen(filename, "w");
      if(fd == NULL){
	ERROR_ "Impossible to create the file %s\n\n",filename _ERROR;

      }
      else{

	/*Alignment part*/

	/*Writing first lines*/
	fprintf(fd,
		"%s%s",
		"##gff-version   3\n",
		"##sequence-region   ");
	/*Writing reference name and size*/
	fprintf(fd,
		"%s  %i  %i\n",
		m->array_of_seqs->seqs[0]->name,
		1,
		m->array_of_seqs->seqs[0]->length);

	/*Writing reference*/
	fprintf(fd,
		"%s\t%s\t%s\t%i\t%i\t%s\t%c\t%s%s\n",
		m->array_of_seqs->seqs[0]->name,
		PACKAGE_NAME,
		"ncRNA",
		m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		".",
		m->sets_of_sequences->sets[i]->ref_interval->strand,
		".\tID=",
		m->array_of_seqs->seqs[0]->name);
	      

	/*Etoile part*/
	if(OPTS_selecting_mode == 0){

	  /*Writing all queries alignment info (take into account the number
	    of sequences wanted by set)*/
	  for(j = 0; (j < m->sets_of_sequences->sets[i]->nb_q_intervals) &&
		(j < nb_seqs_set); j++){

	    temp_index = m->sets_of_sequences->sets[i]->q_intervals[j]->index;

	    fprintf(fd,
		    "%s\t%s\t%s\t%i\t%i\t%f\t%c\t%s%s%s%i\n",
		    m->array_of_seqs->seqs[temp_index]->name,
		    PACKAGE_NAME,
		    "ncRNA",
		    m->sets_of_sequences->sets[i]->q_intervals[j]->pos_start + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->pos_end + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->id_ref,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->strand,
		    ".\tID=",
		    m->array_of_seqs->seqs[temp_index]->name,
		    "_",
		    j + 1);
	  }

	  /*Write fasta sequences*/
	  fprintf(fd, "%s", "##FASTA\n");

	  /*Write reference sequence*/
	  fprintf(fd,
		  ">%s %i %i %c\n%s\n",
		  m->array_of_seqs->seqs[0]->name,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->strand,
		  m->sets_of_sequences->sets[i]->ref_seq);

	  /*Writing all query sequences (take into account the number of 
	    sequences by set*/
	  for(j = 0; (j < m->sets_of_sequences->sets[i]->nb_q_intervals) &&
		(j < nb_seqs_set); j++){

	    temp_index = m->sets_of_sequences->sets[i]->q_intervals[j]->index;

	    fprintf(fd,
		    ">%s%s%i %i %i %c\n%s\n",
		    m->array_of_seqs->seqs[temp_index]->name,
		    "_",
		    j + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->pos_start + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->pos_end + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->strand,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
	  }
	}


	/*Clique search*/
	else{

	  /*Writing all queries alignment info (take into account the number
	    of sequences wanted by set)*/
	  for(j = 0; (j < m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique) &&
		(j < nb_seqs_set); j++){

	    


	    temp_index_clique = m->sets_of_sequences->sets[i]->clique_set->clique_index[j];
	    temp_index_seq = m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->index;


	    fprintf(fd,
		    "%s\t%s\t%s\t%i\t%i\t%f\t%c\t%s%s%s%i\n",
		    m->array_of_seqs->seqs[temp_index_seq]->name,
		    PACKAGE_NAME,
		    "ncRNA",
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->pos_start + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->pos_end + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->id_ref,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->strand,
		    ".\tID=",
		    m->array_of_seqs->seqs[temp_index_seq]->name,
		    "_",
		    j + 1);
	  }

	  /*Write fasta sequences*/
	  fprintf(fd, "%s", "##FASTA\n");

	  /*Write reference sequence*/
	  fprintf(fd,
		  ">%s %i %i %c\n%s\n",
		  m->array_of_seqs->seqs[0]->name,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->strand,
		  m->sets_of_sequences->sets[i]->ref_seq);

	  /*Writing all query sequences (take into account the number of 
	    sequences by set*/
	  for(j = 0; (j < m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique) &&
		(j < nb_seqs_set); j++){

	    temp_index_clique = m->sets_of_sequences->sets[i]->clique_set->clique_index[j];
	    temp_index_seq = m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->index;
	    

	    fprintf(fd,
		    ">%s%s%i %i %i %c\n%s\n",
		    m->array_of_seqs->seqs[temp_index_seq]->name,
		    "_",
		    j + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->pos_start + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->pos_end + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->strand,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->seq);
	  }
	}

	
     
	/*Close file*/
	fclose(fd);
      }
    
      DESTROY(filename);
    }
  }

  return nb_set;

}



////////////////////////////////////////////////////////////////////////////
///
/// \fn int write_multifasta(p_matt_ygraph m)
///
/// \brief Function used to write multifasta files
///
/// \param[in] m : the main structure containing all info needed
///
/// \return the number of files written
///
/// This function will produce as many multifasta as the number of sets of 
/// sequences discovered
///
/////////////////////////////////////////////////////////////////////////////


int write_multifasta(p_matt_ygraph m)
{

  /*Indexes*/
  int i,j;
  /*Current filename*/
  char *filename = NULL;
  /*Size in char of the number of the filename*/
  int size;
  /*Current file descriptor*/
  FILE *fd;
  /*Temporary query index*/
  int temp_index;
  /*Number of sequences by set wanted*/
  int nb_seqs_set = 0;
  /*Number of current set processed*/
  int nb_set = 0;



  /*Check the number of sequences by set wanted : if -1 all seqs wanted*/
  if(OPTS_nb_seqs_set != -1){
    nb_seqs_set = OPTS_nb_seqs_set - 1;
  }
  else{
    nb_seqs_set = 100000;
  }

  /*Get all sets*/
  for(i = 0; i < m->sets_of_sequences->nb_sets; i++){

    /*If no queries : no output*/
    if(((m->sets_of_sequences->sets[i]->nb_q_intervals != 0) && 
	OPTS_selecting_mode == 0) ||
       ((OPTS_selecting_mode != 0) && 
	(m->sets_of_sequences->sets[i]->clique_set != NULL) &&
	(m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique != 0))){

      nb_set++;

      /*Open the new file*/
      get_size_number(nb_set, &size);
      NEW(filename,
	  char,
	  strlen(OPTS_output_dir) + size + strlen(".fna") + 1);
      sprintf(filename,
	      "%s%i%s",
	      OPTS_output_dir,
	      nb_set,
	      ".fna");
      fd = fopen(filename, "w");
      if(fd == NULL){
	ERROR_ "Impossible to create the file %s\n\n",filename _ERROR;

      }
      else{

	/*Etoile part*/
	if(OPTS_selecting_mode == 0){

	  /*Write reference sequence*/
	  fprintf(fd,
		  ">%s %i %i %c\n%s\n",
		  m->array_of_seqs->seqs[0]->name,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->strand,
		  m->sets_of_sequences->sets[i]->ref_seq);
    
	  /*Writing all query sequences (take into account the number of 
	    sequences by set)*/
	  for(j = 0; (j < m->sets_of_sequences->sets[i]->nb_q_intervals) &&
		(j < nb_seqs_set); j++){
      
	    temp_index = m->sets_of_sequences->sets[i]->q_intervals[j]->index;
      
	    fprintf(fd,
		    ">%s%s%i %i %i %c\n%s\n",
		    m->array_of_seqs->seqs[temp_index]->name,
		    "_",
		    j + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->pos_start + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->pos_end + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->strand,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
	  }
    
    
	}
	/*Clique part*/
	else{
	  /*Write reference sequence*/
	  fprintf(fd,
		  ">%s %i %i %c\n%s\n",
		  m->array_of_seqs->seqs[0]->name,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->strand,
		  m->sets_of_sequences->sets[i]->ref_seq);
    
	  /*Writing all query sequences (take into account the number of 
	    sequences by set*/
	  for(j = 0; (j < m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique) &&
		(j < nb_seqs_set); j++){
      
	    temp_index = m->sets_of_sequences->sets[i]->clique_set->clique_index[j];
	    temp_index = m->sets_of_sequences->sets[i]->q_intervals[temp_index]->index;
      
      
	    fprintf(fd,
		    ">%s%s%i %i %i %c\n%s\n",
		    m->array_of_seqs->seqs[temp_index]->name,
		    "_",
		    j + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->pos_start + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->pos_end + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->strand,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
	  }
    
    
	}
	/*Close file*/
	fclose(fd);
      }
    
      DESTROY(filename);
    }
  }

  return nb_set;

}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_process()
///
/// \brief Function used to print starting message on screen
///
/// This function will print on screen the starting message 
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_process()
{
  /*ATTENTION METTRE MODE*/
  PRINT_ LINE _PRINT;
  PRINT_ "\nFIND CONSERVED REGIONS FROM ALIGNMENTS\n" _PRINT;     
  PRINT_ "\n%s\n", LINE _PRINT;

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_sequences()
///
/// \brief Function used to print the starting message on screen concerning
/// the parsing of the sequences
///
/// This function will print the starting message on screen concerning
/// the parsing of the sequences
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_sequences()
{
  PRINT_ "GET REFERENCE FROM FILE : %s\n",OPTS_ref_file _PRINT;
  /*  PRINT_ "GET OTHER SEQUENCES FROM FILE : %s\n\n",OPTS_file _PRINT;*/

}

////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_sequences(sequence_array *array_of_seqs)
///
/// \brief Function used to print the ending message on screen concerning
/// the parsing of the sequences
///
/// This function will print the ending message on screen concerning
/// the parsing of the sequences (number of sequences, their name and length)
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_sequences(sequence_array *array_of_seqs)
{

  /*Index*/
  int i;

  PRINT_ "FOUND %i SEQUENCES\n",array_of_seqs->nb_seqs 
    _PRINT;
  PRINT_ "\tREFERENCE : %s (length : %i)\n",
    array_of_seqs->seqs[0]->name,
    array_of_seqs->seqs[0]->length _PRINT;	
  for(i = 1; i < array_of_seqs->nb_seqs; i++){
    PRINT_ "\tSEQUENCE %i : %s (length : %i)\n",
      i,
      array_of_seqs->seqs[i]->name,
      array_of_seqs->seqs[i]->length _PRINT;
  }
  PRINT_ "\n%s\n", LINE _PRINT;
  
}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_alignments()
///
/// \brief Function used to print the starting message on screen concerning
/// the parsing of the alignments
///
/// This function will print the starting message on screen concerning
/// the parsing of the alignments
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_alignments()
{

  PRINT_ "READ ALIGNMENTS FILES (%i files)\n\n",
    OPTS_alignments->nb_chars _PRINT;

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_alignments_file(char *filename)
///
/// \brief Function used to print the starting message on screen concerning
/// the parsing of the alignments in the current file
///
/// \param[in] filename : the name of the current alignment file
///
/// This function will print the starting message on screen concerning
/// the parsing of the alignments in the current file
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_alignments_file(char *filename)
{
  PRINT_ "\tPARSING ALIGNMENT FILE : %s\n",filename _PRINT;

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_alignments_file(int nb_al)
///
/// \brief Function used to print the starting message on screen concerning
/// the parsing of the alignments in the current file
///
/// \param[in] nb_al : number of alignment found in the current file
///
/// This function will print the ending message on screen concerning
/// the parsing of the alignments in the current file (number of alignments
/// found)
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_alignments_file(int nb_al)
{
  PRINT_ "\t\tAlignments found : %i\n\n",nb_al _PRINT;

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_alignments(int nb_al)
///
/// \brief Function used to print the ending message on screen concerning
/// the parsing of the alignments
///
/// \param[in] nb_al : the number of alignments found
///
/// This function will print the ending message on screen concerning
/// the parsing of the alignments (number of alignments found)
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_alignments(int nb_al)
{

  PRINT_ "ALIGNMENTS FILES READING COMPLETE : %i alignments found\n",
    nb_al _PRINT;
  PRINT_ "\n%s\n", LINE _PRINT;


}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_score()
///
/// \brief Function used to print the starting message on screen concerning
/// the score calculation
///
/// This function will print the starting message on screen concerning
/// the score calculation
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_score()
{

  PRINT_ "CALCULATE SCORE FUNCTION\n\n" _PRINT;
  PRINT_ "\tLAMBDA PARAMETER : %i\n",OPTS_lambda _PRINT;

}

////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_score(float mean)
///
/// \brief Function used to print the ending message on screen concerning
/// the score calculation
///
/// \param[in] mean : the mean number of alignments found by position
///
/// This function will print the ending message on screen concerning
/// the score calculation (mean number of alignments)
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_score(float mean)
{
  PRINT_ "\tMEAN NUMBER OF ALIGNMENTS BY POSITION : %f\n\n",mean _PRINT;
  PRINT_ "SCORE FUNCTION CALCULATED\n" _PRINT;
  
  PRINT_ "\n%s\n", LINE _PRINT;

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_conserved_regions()
///
/// \brief Function used to print the starting message on screen concerning
/// the searching for conserved regions
///
/// This function will print the starting message on screen concerning
/// the searching of the conserved regions
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_conserved_regions()
{
  PRINT_ "FINDING CONSERVED REGIONS\n\n" _PRINT;
  PRINT_ "\tMININUM LENGTH OF A REGION : %i\n",
    OPTS_min_length_region _PRINT;
  PRINT_ "\tMAXIMUM LENGTH OF A REGION : %i\n\n",
    OPTS_max_length_region _PRINT;
  
}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_conserved_regions(int nb_sets)
///
/// \brief Function used to print the ending message on screen concerning
/// the searching for conserved regions
///
/// \param[in] nb_sets : the number of conserved regions found
///
/// This function will print the ending message on screen concerning
/// the searching of the conserved regions (number of regions found)
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_conserved_regions(int nb_sets)
{

  PRINT_ "CONSERVED REGIONS FOUND : %i\n",nb_sets _PRINT;
  PRINT_ "\n%s\n", LINE _PRINT;

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_build_sets()
///
/// \brief Function used to print the starting message on screen concerning
/// the building sets step
///
/// This function will print the starting message on screen concerning
/// the building sets step
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_build_sets()
{

  PRINT_ "BUILDING SETS OF SEQUENCES\n\n" _PRINT;

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_build_sets(int nb_sets)
///
/// \brief Function used to print the ending message on screen concerning
/// the building sets step
///
/// \param[in] nb_sets : the number of sets built
///
/// This function will print the ending message on screen concerning
/// the building sets step (number of sets built)
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_build_sets(int nb_sets)
{

  PRINT_ "SETS OF SEQUENCES BUILT : %i\n",nb_sets _PRINT;
  PRINT_ "\n%s\n", LINE _PRINT;
  
}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_compare_ref()
///
/// \brief Function used to print the starting message on screen concerning
/// the comparison step of all queries against the reference
///
/// This function will print the starting message on screen concerning
/// the comparison step of all queries against the reference
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_compare_ref()
{
  PRINT_ "COMPARING ALL QUERIES AGAINST THE REFERENCE AND ELIMINATE REDUNDANCY\n\n" _PRINT;
  PRINT_ "\tMINIMUM IDENTITY THRESHOLD : %i\n", OPTS_min_id _PRINT;
  PRINT_ "\tMAXIMUM IDENTITY THRESHOLD : %i\n", OPTS_max_id _PRINT;

  if(OPTS_carnac){
    PRINT_ "\tCOMMON SECONDARY STRUCTURE SHARED\n\n" _PRINT;
  }
  else{
    PRINT_ "\n" _PRINT;
  }

  
}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_compare_ref_set(int nb_set, int nb_seqs)
///
/// \brief Function used to print the starting message on screen concerning
/// the comparison step of all queries against the reference for the current
/// set
///
/// \param[in] nb_set : the number of set processed
/// \param[in] nb_seqs : the number of sequences in the set (ref + queries)
///
/// This function will print the starting message on screen concerning
/// the comparison step of all queries against the reference for the current
/// set (nb_set). It also print the number of sequences in the set
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_compare_ref_set(int nb_set, int nb_seqs)


{
  
  PRINT_ "\tSET %i (%i sequences) -> ",nb_set,nb_seqs _PRINT; 

}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_compare_ref_set(int nb_seqs)
///
/// \brief Function used to print the ending message on screen concerning
/// the comparison step of all queries against the reference for the current
/// set
///
/// \param[in] nb_seqs : the number of seqs in the set after the comparison
/// (ref + query)
///
/// This function will print the ending message on screen concerning
/// the comparison step of all queries against the reference for the current
/// set
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_compare_ref_set(int nb_seqs)
{

  if(nb_seqs != 1){
    PRINT_ " %i sequences left\n",nb_seqs _PRINT;
  }
  else{
    PRINT_ " No connected sequence\n" _PRINT;
  }


}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_sort_identity_ref()
///
/// \brief Function used to print the message on screen concerning the
/// sorting of queries by identities
///
/// This function will print the message on screen concerning the
/// sorting of queries by identities
///
/////////////////////////////////////////////////////////////////////////////


extern void print_verbose_sort_identity_ref()
{

  PRINT_ "\nSORTING QUERIES IN SETS BY REFERENCE IDENTITY...\n" _PRINT;

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_compare_ref()
///
/// \brief Function used to print the ending message on screen concerning
/// the comparison step of all queries against the reference
///
/// This function will print the ending message on screen concerning
/// the comparison step of all queries against the reference
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_compare_ref()
{

  PRINT_ "\nCOMPARISONS DONE\n" _PRINT;
  PRINT_ "\n%s\n", LINE _PRINT;
}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_check_repetitions()
///
/// \brief Function used to print the message on screen concerning the
/// checking of species repetitions
///
/// This function will print the message on screen concerning the
/// checking of species repetitions
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_check_repetitions()
{
  PRINT_ "\nCHECKING SPECIES REPETITIONS\n\n" _PRINT;
  PRINT_ "\tMAXIMUM NUMBER OF SPECIES REPETITIONS : %i\n\n", 
    OPTS_max_species_repet _PRINT;

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_checking_repetitions_set(int nb_set, int nb_seqs)
///
/// \brief Function used to print the starting message on screen concerning
/// the species repetitions for the current set
///
/// \param[in] nb_set : the number of set processed
/// \param[in] nb_seqs : the number of sequences in the set (ref + queries)
///
/// This function will print the starting message on screen concerning
/// the species repetitions for the current set (nb_set). 
/// It also print the number of sequences in the set
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_checking_repetitions_set(int nb_set, int nb_seqs)


{
  
  PRINT_ "\tSET %i (%i sequences) -> ",nb_set,nb_seqs _PRINT; 

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_checking_repetitions_set(int nb_seqs)
///
/// \brief Function used to print the ending message on screen concerning
/// the species repetitions for the current set
///
/// \param[in] nb_seqs : the number of sequences in the set (ref + queries)
///
/// This function will print the ending message on screen concerning
/// the species repetitions for the current set by printing the number of 
/// sequences in the set
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_checking_repetitions_set(int nb_seqs)
{
  if(nb_seqs != 1){
    PRINT_ " OK\n" _PRINT;
  }
  else{
    PRINT_ " Species repetitions exceeded\n" _PRINT;
  }


}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_check_repetitions()
///
/// \brief Function used to print the ending message on screen concerning the
/// checking of species repetitions
///
/// This function will print the ending message on screen concerning the
/// checking of species repetitions
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_check_repetitions()
{

  PRINT_ "CHECKING SPECIES REPETITIONS DONE\n\n" _PRINT;


}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_search_naive_clique()
///
/// \brief Function used to print the starting message on screen concerning
/// the comparison step of the naive clique search
///
/// This function will print the starting message on screen concerning
/// the comparison step of of the naive clique search
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_search_naive_clique()
{
  PRINT_ "SEARCHING NAIVE CLIQUE\n\n" _PRINT;
  PRINT_ "\tMINIMUM IDENTITY THRESHOLD : %i\n", OPTS_min_id _PRINT;
  PRINT_ "\tMAXIMUM IDENTITY THRESHOLD : %i\n", OPTS_max_id _PRINT;
  if(OPTS_nb_seqs_set != -1){
    PRINT_ "\tMAXIMUM NUMBER OF SEQUENCES BY SET WANTED : %i\n",
      OPTS_nb_seqs_set _PRINT;
  }
  else{
    PRINT_ "\tMAXIMUM NUMBER OF SEQUENCES BY SET WANTED : ALL\n" _PRINT;

  }
  if(OPTS_carnac){
    PRINT_ "\tCOMMON SECONDARY STRUCTURE SHARED\n\n" _PRINT;
  }
  else{
    PRINT_ "\n" _PRINT;

  }
}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_naive_clique_set(int nb_set, int nb_seqs)
///
/// \brief Function used to print the starting message on screen concerning
/// the comparison step of searching the naive clique for the current
/// set
///
/// \param[in] nb_set : the number of set processed
/// \param[in] nb_seqs : the number of sequences in the set (ref + queries)
///
/// This function will print the starting message on screen concerning
/// the comparison step of searching the naive clique for the current
/// set (nb_set). It also print the number of sequences in the set
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_naive_clique_set(int nb_set, int nb_seqs)
{

  PRINT_ "\tSET %i (%i sequences) -> ",nb_set,nb_seqs _PRINT; 


}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_naive_clique_set(int nb_seqs)
///
/// \brief Function used to print the ending message on screen concerning
/// the comparison step of searching the naive clique for the current
/// set
///
/// \param[in] nb_seqs : the number of seqs in the set after the naive
/// clique serach (ref + query)
///
/// This function will print the ending message on screen concerning
/// the comparison step of searching the naive clique for the current
/// set
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_naive_clique_set(int nb_seqs)
{

  if(nb_seqs != 0){
    PRINT_ " %i sequences clique\n",nb_seqs _PRINT;
  }
  else{
    PRINT_ " No clique found\n" _PRINT;
  }

}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_search_naive_clique()
///
/// \brief Function used to print the ending message on screen concerning
/// the comparison step of the naive clique search
///
/// This function will print the ending message on screen concerning
/// the comparison step of of the naive clique search
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_search_naive_clique()
{
  PRINT_ "NAIVE CLIQUE SEARCH DONE\n" _PRINT;
  PRINT_ "\n%s\n", LINE _PRINT;
}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_no_connect(int nb)
///
/// \brief Function used to print the the message when a set of sequences
/// has no connected sequence
///
/// \param nb : the number of the set concerned
///
/// This function will print the the message when a set of sequences
/// has no connected sequence
///
/////////////////////////////////////////////////////////////////////////////

void print_verbose_no_connect(int nb)
{

  PRINT_ "\tSET %i : no connected sequence with the reference\n",nb _PRINT;

}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_search_naive_clique()
///
/// \brief Function used to print the starting message on screen concerning
/// the comparison step of the maxi clique search
///
/// This function will print the starting message on screen concerning
/// the comparison step of of the maxi clique search
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_search_maxi_clique()
{
  PRINT_ "SEARCHING MAXI CLIQUE\n\n" _PRINT;
  PRINT_ "\tMINIMUM IDENTITY THRESHOLD : %i\n", OPTS_min_id _PRINT;
  PRINT_ "\tMAXIMUM IDENTITY THRESHOLD : %i\n", OPTS_max_id _PRINT;
  if(OPTS_nb_seqs_set != -1){
    PRINT_ "\tMAXIMUM NUMBER OF SEQUENCES BY SET WANTED : %i\n",
      OPTS_nb_seqs_set _PRINT;
  }
  else{
    PRINT_ "\tMAXIMUM NUMBER OF SEQUENCES BY SET WANTED : ALL\n" _PRINT;

  }
  if(OPTS_carnac){
    PRINT_ "\tCOMMON SECONDARY STRUCTURE SHARED\n\n" _PRINT;
  }
  else{
    PRINT_ "\n" _PRINT;

  }



}

////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_maxi_clique_set(int nb_set, int nb_seqs)
///
/// \brief Function used to print the starting message on screen concerning
/// the comparison step of searching the maxi clique for the current
/// set
///
/// \param[in] nb_set : the number of set processed
/// \param[in] nb_seqs : the number of sequences in the set (ref + queries)
///
/// This function will print the starting message on screen concerning
/// the comparison step of searching the maxi clique for the current
/// set (nb_set). It also print the number of sequences in the set
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_maxi_clique_set(int nb_set, int nb_seqs)
{

  PRINT_ "\tSET %i (%i sequences) -> ",nb_set,nb_seqs _PRINT; 

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_maxi_clique_set(int nb_seqs)
///
/// \brief Function used to print the ending message on screen concerning
/// the comparison step of searching the maxi clique for the current
/// set
///
/// \param[in] nb_seqs : the number of seqs in the set after the maxi
/// clique serach (ref + query)
///
/// This function will print the ending message on screen concerning
/// the comparison step of searching the maxi clique for the current
/// set
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_maxi_clique_set(int nb_seqs)
{
  if(nb_seqs != 0){
    PRINT_ " %i sequences clique\n",nb_seqs _PRINT;
  }
  else{
    PRINT_ " No clique found\n" _PRINT;
  }

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_search_naive_clique()
///
/// \brief Function used to print the ending message on screen concerning
/// the comparison step of the maxi clique search
///
/// This function will print the ending message on screen concerning
/// the comparison step of of the maxi clique search
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_search_maxi_clique()
{
  PRINT_ "MAXI CLIQUE SEARCH DONE\n" _PRINT;
  PRINT_ "\n%s\n", LINE _PRINT;

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_write_GFF3()
///
/// \brief Function used to print the starting message on screen concerning
/// the writing of result GFF3 files
///
/// This function will print the starting message on screen concerning
/// the writing of result GFF3 files
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_write_GFF3()
{
  PRINT_ "WRITE SETS OF SEQUENCES IN GFF3 format\n\n" _PRINT;
  PRINT_ "\tOUTPUT DIR : %s\n",OPTS_output_dir _PRINT;
  if(OPTS_nb_seqs_set != -1){
    PRINT_ "\tNUMBER OF SEQUENCES BY SET : %i\n\n",OPTS_nb_seqs_set _PRINT;
  }
  else{
    PRINT_ "\tNUMBER OF SEQUENCES BY SET : ALL\n\n" _PRINT;
  }
}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_write_GFF3(int nb_files)
///
/// \brief Function used to print the ending message on screen concerning
/// the writing of result GFF3 files
///
/// \param[in] nb_files : the number of GFF3 files written
///
/// This function will print the ending message on screen concerning
/// the writing of result GFF3 files (number of files written)
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_write_GFF3(int nb_files)
{

  PRINT_ "GFF3 FILES WRITTEN : %i\n",nb_files  _PRINT; 
  PRINT_ "\n%s\n", LINE _PRINT;
}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_start_write_multifasta()
///
/// \brief Function used to print the starting message on screen concerning
/// the writing of result multifasta files
///
/// This function will print the starting message on screen concerning
/// the writing of result multifasta files
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_start_write_multifasta()
{
  PRINT_ "WRITE SETS OF SEQUENCES IN MULTIFASTA FORMAT\n\n" _PRINT;
  PRINT_ "\tOUTPUT DIR : %s\n",OPTS_output_dir _PRINT;
  if(OPTS_nb_seqs_set != -1){
    PRINT_ "\tNUMBER OF SEQUENCES BY SET : %i\n\n",OPTS_nb_seqs_set _PRINT;
  }
  else{
    PRINT_ "\tNUMBER OF SEQUENCES BY SET : ALL\n\n" _PRINT;
  }


}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_write_multifasta(int nb_files)
///
/// \brief Function used to print the ending message on screen concerning
/// the writing of result multifasta files
///
/// \param[in] nb_files : the number of multifasta files written
///
/// This function will print the ending message on screen concerning
/// the writing of result multifasta files (number of files written)
///
/////////////////////////////////////////////////////////////////////////////


void print_verbose_end_write_multifasta(int nb_files)
{

  PRINT_ "MULTIFASTA FILES WRITTEN : %i\n",nb_files  _PRINT; 
  PRINT_ "\n%s\n", LINE _PRINT;

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void print_verbose_end_process(long execution_time)
///
/// \brief Function used to print the ending message
///
/// \param[in] execution_time : the execution time spent
///
/// This function will print the ending message (with execution time)
///
/////////////////////////////////////////////////////////////////////////////

void print_verbose_end_process(long execution_time){

 PRINT_ "PROCESS FINISHED\n" _PRINT;

 /*Execution time < 1s : print time in milliseconds*/
 if(execution_time <= 1000){

   PRINT_ "\tEXECUTION TIME : %ldms\n\n",execution_time _PRINT;

 }
 /**Execution time > 1s : print time in seconds and milliseconds*/
 else{
   
   PRINT_ "\tEXECUTION TIME : %lis%ldms\n\n",execution_time/1000,execution_time%1000 _PRINT;


 }


}
