/////////////////////////////////////////////////////////////////////////////
///
/// \file main.c
///
/// \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
///
/// \brief File that defines the main software function
///
/// \version 1.02
///
/// \date September 2008
///
///
/// This file represents the main entrance of the software and contains the
/// main software algorithm
///
///
/////////////////////////////////////////////////////////////////////////////


/*Basic Inclusions*/
#include <sys/time.h>

/*Intern inclusions*/
#include "io.h"
#include "types.h"
#include "macros.h"
#include "options.h"
#include "sequence.h"
#include "alignment.h"
#include "matt_ygraph.h"


/////////////////////////////////////////////////////////////////////////////
///
/// \fn int main(int argc, char **argv)
///
/// \brief Main function
///
/// \param[in] argc : number of parameters
/// \param[in] argv : the parameters
///
/// \return an integer representing the main exit status
///
/// This function contains the main step algorithm of the software
///
///
/////////////////////////////////////////////////////////////////////////////


int main(int argc, char **argv) 
{
 
  /*Status*/
  int status = 1;
  /*Number of result files*/
  int nb_files = 0;


  /*Start time of the process*/
  struct timeval *start_time = NULL;
  /*End time of the process*/
  struct timeval *end_time = NULL;
  /*Execution time spent in milliseconds*/
  long execution_time;
  

  /*Main sofwtare structure that will contain all information*/
  p_matt_ygraph matt_ygraph = NULL;

  NEW(start_time, struct timeval, 1);
  NEW(end_time, struct timeval, 1);

  /*Get start time*/
  gettimeofday(start_time,NULL);
  

  /*Get the parameters*/
  status = parse_params(argc, argv);

  /*Check the returned status*/
  if(status == 2){
    usage();
  }
  else{

    /*Check parameters validity*/
    if(!check_parameters_validity()){
      usage();
    }
    else{

      /*Create the main structure*/
      NEW(matt_ygraph, matt_ygraph_s, 1);
      NEW(matt_ygraph->array_of_seqs, sequence_array, 1);
      NEW(matt_ygraph->array_of_al_info, alignment_info_array,1);
      NEW(matt_ygraph->array_of_scores, score_array, 1);
      NEW(matt_ygraph->interesting_regions, interval_array, 1);
      NEW(matt_ygraph->sets_of_sequences, sets_of_seqs_array, 1);

      matt_ygraph->array_of_seqs->nb_seqs = 0;
      matt_ygraph->array_of_al_info->nb_al_info = 0;
      matt_ygraph->interesting_regions->nb_intervals = 0;
      matt_ygraph->sets_of_sequences->nb_sets = 0;

      if(OPTS_verbose){
	print_verbose_start_process();
	print_verbose_start_sequences();
      }
     

      /*Get the sequences from the file*/
      get_sequences(matt_ygraph->array_of_seqs);

      /*Complete unknown attributes like species, put the sequence
       in uppercase, replace "space" by "_" in sequence name etc.*/
      complete_sequences_attributes(matt_ygraph->array_of_seqs);

      /*Sequence parser error -> stop*/
      if(matt_ygraph->array_of_seqs->seqs != NULL){
	
	if(OPTS_verbose){
	  print_verbose_end_sequences(matt_ygraph->array_of_seqs);
	}

	if(OPTS_verbose){
	  print_verbose_start_alignments();
	}
	
	/*Read the alignments file(s) and get the information*/
	read_alignments_files(matt_ygraph);

	/*If alignments were found*/
	if(matt_ygraph->array_of_al_info->al_info != NULL){

	  /*Complete missing info about alignments (strand, query/subject,
	    indexes and sorting)*/
	  complete_alignments_attributes(matt_ygraph->array_of_al_info,
					 matt_ygraph->array_of_seqs);

	  if(OPTS_verbose){
	    print_verbose_end_alignments(matt_ygraph->array_of_al_info->
					 nb_al_info);
	    print_verbose_start_score();
	  }

	  /*Calculate the number of alignments by position*/
	  NEW(matt_ygraph->array_of_scores->positions,
	      int,
	      matt_ygraph->array_of_seqs->seqs[0]->length);
	  matt_ygraph->array_of_scores->length =
	    matt_ygraph->array_of_seqs->seqs[0]->length;

	  calculate_nb_al_function(matt_ygraph);


	  /*Calculate the score function*/
	  NEW(matt_ygraph->array_of_scores->scores,
	      float,
	      matt_ygraph->array_of_scores->length);
	  calculate_score_function(matt_ygraph);
	  
	  if(OPTS_verbose){
	    print_verbose_end_score(matt_ygraph->array_of_scores->mean_al);
	    print_verbose_start_conserved_regions();

	  }

	  /*Determine the intervals of interesting regions*/
	  get_interesting_regions(matt_ygraph);

	  if(OPTS_verbose){
	    print_verbose_end_conserved_regions(matt_ygraph->
						interesting_regions->
						nb_intervals);
	    print_verbose_start_build_sets();
	  }


	  /*Get the corresponding sets of sequences*/
	  get_sets_of_sequences(matt_ygraph);
	  
	  if(OPTS_verbose){
	    print_verbose_end_build_sets(matt_ygraph->
					 sets_of_sequences->
					 nb_sets);
	  }

	  /*Compare for all sets, the queries against the reference
	    to refine the bounds and eliminate those who don't respect
	    the identity thresholds*/

	  if(OPTS_verbose){
	    print_verbose_start_compare_ref();
	  }

	  compare_all_against_ref(matt_ygraph);

	  
	  if(OPTS_verbose){
	    print_verbose_sort_identity_ref();
	  }

	  sort_identity_ref(matt_ygraph);

  
	  /*Check species repetitions if wanted*/
	  if(OPTS_max_species_repet != -1){
	    if(OPTS_verbose){
	      print_verbose_start_check_repetitions();
	    }

	    check_species_repetitions(matt_ygraph);

	    if(OPTS_verbose){
	      print_verbose_end_check_repetitions();
	    }
	    
	  }

	  if(OPTS_verbose){
	    print_verbose_end_compare_ref();
	  }


	  /*Naive clique search*/
	  if(OPTS_selecting_mode == 1){
	    
	    if(OPTS_verbose){
	      print_verbose_start_search_naive_clique();
	    }

	    search_naive_clique(matt_ygraph);

	    if(OPTS_verbose){
	      print_verbose_end_search_naive_clique();
	    }


	  }
	  else{
	    /*Maxi clique search*/
	    if(OPTS_selecting_mode == 2){
	      
	      if(OPTS_verbose){
		print_verbose_start_search_maxi_clique();
	      }

	      maxi_clique_search(matt_ygraph);


	      if(OPTS_verbose){
		print_verbose_end_search_maxi_clique();
	      }
	    }
	  }
	  /*Write GFF3 output*/
	  if(OPTS_output_format == 1){
	  
	    if(OPTS_verbose){
	      print_verbose_start_write_GFF3();

	    }

	    nb_files = write_GFF3(matt_ygraph);
  
	    if(OPTS_verbose){
	      print_verbose_end_write_GFF3(nb_files);
	    }
	  }
	  /*Multifasta output*/
	  else{
	    if(OPTS_verbose){
	      print_verbose_start_write_multifasta();
	      
	    }
	    
	    nb_files = write_multifasta(matt_ygraph);
	    
	    if(OPTS_verbose){
	      print_verbose_end_write_multifasta(nb_files);
	    }



	  }


	  if(OPTS_verbose){

	    gettimeofday(end_time, NULL);

	    /*Execution time calculation*/
	    execution_time = ((end_time->tv_sec - start_time->tv_sec) * 
			      1000L) + 
	      ((end_time->tv_usec - start_time->tv_usec)/1000L) ;  
	    
	    print_verbose_end_process(execution_time);
	  }


	}

      }

    }

    /*Destroy main structure*/
    destroy_main_structure(matt_ygraph);
    /*Destroy options*/
    destroy_options();

    
   
  }

  DESTROY(start_time);
  DESTROY(end_time);

  return status;


}

