////////////////////////////////////////////////////////////////////////////
///
/// \file matt_ygraph.c
///
/// \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
///
/// \brief File that all functions used to retrieve conserved regions
/// and select some
///
/// \version 1.02
///
/// \date September 2008
///
///
/// This file will define all functions that will retrieve conserved regions
/// according to alignments and all functions used to select sequences
/// according to users parameters
///
///
/////////////////////////////////////////////////////////////////////////////

/*Basic inclusions*/
#include <math.h>


/*Intern inclusions*/
#include "matt_ygraph.h"
#include "sequence.h"
#include "macros.h"
#include "alignment.h"
#include "options.h"
#include "io.h"
#include "carnac.h"




////////////////////////////////////////////////////////////////////////////
///
/// \fn void destroy_main_structure(p_matt_ygraph m)
/// 
/// \brief Function used to destroy all structures allocated
///
/// \param[in,out] m : the main structure to destroy
///
/// That function will destroy all allocated memory according to mode used
///
/////////////////////////////////////////////////////////////////////////////


void destroy_main_structure(p_matt_ygraph m)
{

  /*Indexes*/
  int i,j;

  if(m){
  
    /*Destroy the sequences*/
    if(m->array_of_seqs){
      for(i = 0; i < m->array_of_seqs->nb_seqs; i++) {
	destroy_sequence(m->array_of_seqs->seqs[i]);
      }
      DESTROY(m->array_of_seqs->seqs);
      DESTROY(m->array_of_seqs);
    
    }
  
    /*Destroy the alignments*/
    if(m->array_of_al_info){
      for(i = 0; i < m->array_of_al_info->nb_al_info; i++){
	destroy_alignment(m->array_of_al_info->al_info[i]);
      }
      DESTROY(m->array_of_al_info->al_info);
      DESTROY(m->array_of_al_info);

    }

    /*Destroy the scores*/
    if(m->array_of_scores){
      if(m->array_of_scores->positions){
	DESTROY(m->array_of_scores->positions);
      }
      if(m->array_of_scores->scores){
	DESTROY(m->array_of_scores->scores);
      }
      DESTROY(m->array_of_scores);
    }


    /*Destroy interesting regions*/
    if(m->interesting_regions){
      if(m->interesting_regions->intervals){
	for(i = 0; i < m->interesting_regions->nb_intervals; i++){
	  if(m->interesting_regions->intervals[i]){
	    DESTROY(m->interesting_regions->intervals[i]);
	  }
	}
	DESTROY(m->interesting_regions->intervals);
      }
      DESTROY(m->interesting_regions);
    }

    /*Destroy sets of sequences*/
    if(m->sets_of_sequences){
      if(m->sets_of_sequences->sets){
	for(i = 0; i < m->sets_of_sequences->nb_sets; i++){
	  for(j = 0; j < m->sets_of_sequences->sets[i]->nb_q_intervals; j++){
	    if(m->sets_of_sequences->sets[i]->q_intervals[j]){
	      if(m->sets_of_sequences->sets[i]->q_intervals[j]->seq != NULL){
		DESTROY(m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
	      }
	      DESTROY(m->sets_of_sequences->sets[i]->q_intervals[j]);
	    }
	  }	
	  DESTROY(m->sets_of_sequences->sets[i]->q_intervals);
	  DESTROY(m->sets_of_sequences->sets[i]->ref_interval);
	  if(m->sets_of_sequences->sets[i]->ref_seq != NULL){
	    DESTROY(m->sets_of_sequences->sets[i]->ref_seq);
	  }

	  /*Destroying clique if built*/
	  if(m->sets_of_sequences->sets[i]->clique_set){
	    if(m->sets_of_sequences->sets[i]->clique_set->clique_index){
	      DESTROY(m->sets_of_sequences->sets[i]->clique_set->clique_index);
	    }
	    DESTROY(m->sets_of_sequences->sets[i]->clique_set);
	  }

	  /*Destroying neighbors if built*/
	  if(m->sets_of_sequences->sets[i]->neighbors){
	    for(j = 0; j < m->sets_of_sequences->sets[i]->nb_q_intervals; j++){
	      if(m->sets_of_sequences->sets[i]->neighbors[j]){
		if(m->sets_of_sequences->sets[i]->neighbors[j]->indexes){
		  DESTROY(m->sets_of_sequences->sets[i]->neighbors[j]->indexes);
		}
		DESTROY(m->sets_of_sequences->sets[i]->neighbors[j]);
	      }
	    }
	    DESTROY(m->sets_of_sequences->sets[i]->neighbors);
	  }

	
	  DESTROY(m->sets_of_sequences->sets[i]);
	}
	DESTROY(m->sets_of_sequences->sets);    
      }
      DESTROY(m->sets_of_sequences);
    }


    /*Destroy the regexp of carnac if used*/
    if(OPTS_carnac){
      destroy_regex();
    }


    /*Destroy the mattress object itself*/
    DESTROY(m);
  }
}


////////////////////////////////////////////////////////////////////////////
///
/// \fn int is_present_int_array(int num, int *array, int length)
/// 
/// \brief Function used to check if a number is present in an array of
/// integers
///
/// \param[in] num : the number to search for
/// \param[in] array : the array of integers
/// \param[in] length : length of the array
///
/// \return 1 if the number was found, 0 otherwise
///
/// That function will check if a number is present in an array of integres
/// and return 1 if  it was found and 0 in the other case
///
/////////////////////////////////////////////////////////////////////////////

int is_present_int_array(int num, int *array, int length){
  
  /*Index*/
  int i; 

  /*Search over the array*/
  for(i = 0; i < length; i++){
    if(num == array[i]){
      return 1;
    }
  }

  return 0;
}


////////////////////////////////////////////////////////////////////////////
///
/// \fn int is_present_char_array(char *string, char_array *array)
/// 
/// \brief Function used to check if a string is present in a char_array
///
/// \param[in] string : the string to search for
/// \param[in] array : the char_array
///
/// \return 1 if the string was found, 0 otherwise
///
/// That function will check if a string is present in an array of strings
/// (char_array) and return 1 if  it was found and 0 in the other case
///
/////////////////////////////////////////////////////////////////////////////

int is_present_char_array(char *string, char_array *array){
  
  /*Index*/
  int i; 

  /*Search over the array*/
  for(i = 0; i < array->nb_chars; i++){
    if(strcmp(string,array->chars[i]) == 0){
      return 1;
    }
  }

  return 0;
}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void copy_interval_query(p_interval_query interval_1,
///                              p_interval_query interval_2)
/// 
/// \brief Function used to copy two interval queries
///
/// \param[in,out] interval_1 : the interval to copy in
/// \param[in] interval_2 : the interval to copy
///
/// That function will copy all the attributes of the interavl_2 in the 
/// first one
///
/////////////////////////////////////////////////////////////////////////////

void copy_interval_query(p_interval_query interval_1,
			 p_interval_query interval_2){



  interval_1->pos_start = interval_2->pos_start;
  interval_1->pos_end = interval_2->pos_end;
  interval_1->index = interval_2->index;
  NEW(interval_1->seq,
      char,
      strlen(interval_2->seq) + 1);
  strcpy(interval_1->seq, interval_2->seq);
  interval_1->strand = interval_2->strand;
  interval_1->id_ref = interval_2->id_ref;
  

}




///////////////////////////////////////////////////////////////////////////
///
/// \fn float score_function(float score_pos_i_minus_1,
///                          int nb_al_pos_i,
///                          float mean_al,
///                          float epsilon,
///                          int lambda)
///
/// \brief Defines the score function for the position i
///
/// \param[in] score_pos_i_minus_1 : the score at position i - 1
/// \param[in] nb_al_pos_i : number of alignments at position i
/// \param[in] mean_al : mean number of alignments by position
/// \param[in] epsilon : float to add at the nb_al_i parameter to avoid
/// log(0)
/// \param[in] lambda : parameter threshold
///
/// \return The score at the position i
///
/// \note Si = max(0,Si-1 + log(ni + espilon / lanmba A)
///          Si : score at position i
///          Si-1 : score at position i-1
///          ni : number of alignments at position i$
///          epsilon : float to avoid log(0) in case ni = 0
///          A : mean number of alignments
///          lambda : parameter threshold
///
/// This function will calculate the score by position
///
///////////////////////////////////////////////////////////////////////////


float score_function(float score_pos_i_minus_1,		
		     int  nb_al_pos_i,			
		     float  mean_al,				
		     float  epsilon,				
		     int  lambda)
{	

  /*Temporary score*/
  float temp_score;
  /*Temporary score used in the log calculation*/
  float log_score;
  /*Score that will be calculated*/
  float score = 0.0;

  /*Use epsilon only if nb_al_pos_i = 0*/
  if(nb_al_pos_i != 0){
    epsilon = 0;
  }

  temp_score = ((float)nb_al_pos_i + epsilon) / (((float)OPTS_lambda) * mean_al);
  log_score = log10(temp_score);
			
  score = (float)MAX(0.0, score_pos_i_minus_1 + log_score); 

  return score;


}




////////////////////////////////////////////////////////////////////////////
///
/// \fn void calculate_nb_al_function(p_matt_ygraph m)
/// 
/// \brief Function used to calculate the number of alignments by position
///
/// \param[in,out] m : the main structure containing all needed information
///
/// That function will count the number of alignments by position and fill
/// the corresponding array
///
/////////////////////////////////////////////////////////////////////////////

void calculate_nb_al_function(p_matt_ygraph m){

  /*Indexes*/
  int i,j;
  /*Current alignment starting position on the reference*/
  int start_pos = 0;
  /*Current alignment ending position on the reference*/
  int end_pos = 0;
  /*Temporay integer used in the calculation of the mean number 
    of alignments by position*/
  int nb_al = 0;
  /*Non redundant array of alignments if the species filter is chosen*/
  alignment_info_array *nr_al_info = NULL;

  /*No filter case : all alignments taken into account*/
  if(!OPTS_filter_species){

    /*Get all alignment positions on reference*/
    for(i = 0; i < m->array_of_al_info->nb_al_info; i++){

      start_pos = m->array_of_al_info->al_info[i]->pos_start_ref;
      end_pos = m->array_of_al_info->al_info[i]->pos_end_ref;

      /*Increase the number of alignments between these positions*/
      for(j = start_pos; j <= end_pos; j++){
	nb_al++;
	m->array_of_scores->positions[j]++;

      }
    
    }   
  }

  /*Filter redundant alignments intra species*/
  else{

    /*Get a copy of the array of alignments but non redundant intra species*/
    NEW(nr_al_info, alignment_info_array, 1);
    eliminate_redundancy_species(m, nr_al_info);

    /*Get all alignment positions on reference*/
    for(i = 0; i < nr_al_info->nb_al_info; i++){

      start_pos = nr_al_info->al_info[i]->pos_start_ref;
      end_pos = nr_al_info->al_info[i]->pos_end_ref;

      /*Increase the number of alignments between these positions*/
      for(j = start_pos - 1; j < end_pos; j++){
	nb_al++;
	m->array_of_scores->positions[j]++;

      }
    
    }


    /*Destroy the nr array*/
    for(i = 0; i < nr_al_info->nb_al_info; i++){
      destroy_alignment(nr_al_info->al_info[i]);
    }
    DESTROY(nr_al_info->al_info);
    DESTROY(nr_al_info);
    
    
  }
  
  /*Calculate the mean number of alignments according to the real length
    of the reference sequence*/
  m->array_of_scores->mean_al = ((float)nb_al / 
				 (float)m->array_of_seqs->seqs[0]->real_length);
}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void calculate_score_function(p_matt_ygraph m)
/// 
/// \brief Function used to calculate the score function
///
/// \param[in,out] m : the main structure containing information
///
/// That function will calculate the score function used to retrieve
/// conserved region. The score function is defined in the macros.h file
///
/////////////////////////////////////////////////////////////////////////////

void calculate_score_function(p_matt_ygraph m)
{

  /*Indexes*/
  int i; 
  /*Score for the current position*/
  float current_score;
  /*Epsilon used in the score array*/
  float epsilon = 0.001;
  /*Mean number of alignments*/
  float mean_al;
  /*Maximum score threshold*/
  float max_score = 20.0;


  /*Retrieve the mean number of alignments*/
  mean_al = m->array_of_scores->mean_al;

  /*Calculate the score for the first position*/
  current_score = score_function(0.0,
				 m->array_of_scores->positions[0],
				 mean_al,
				 epsilon,
				 OPTS_lambda);
  if(current_score > max_score){
    current_score = max_score;
  }
  m->array_of_scores->scores[0] = current_score;

  /*Calculate the score for all positions*/
  for(i = 1; i < m->array_of_scores->length; i++){
    current_score = score_function(m->array_of_scores->scores[i - 1],
				   m->array_of_scores->positions[i],
				   mean_al,
				   epsilon,
				   OPTS_lambda);


/*     if(current_score > max_score){ */
/*       current_score = max_score; */
/*     } */

    m->array_of_scores->scores[i] = current_score;
  }

}




////////////////////////////////////////////////////////////////////////////
///
/// \fn void get_interesting_regions(p_matt_ygraph m)
///
/// \brief Function used to determine interesting regions on the reference
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated with defined regions
///
/// That function will use the scores calculated by the score function to
/// determine positions of conserved regions on the reference sequence
///
/////////////////////////////////////////////////////////////////////////////

void get_interesting_regions(p_matt_ygraph m)
{

  /*Index*/
  int i;
  /*Start position of the current new region*/
  int start_position = -1;
  /*End position of the current new region*/
  int end_position = -1;
  /*Local score maximum*/
  float local_max = -1.0;
  /*Index of the local score maximum*/
  int index_local_max = 0;
  /*Current score*/
  float current_score;

  /*Search for positive scores in the array of scores*/
  for(i = 0; i < m->array_of_scores->length; i++){
    
    current_score = m->array_of_scores->scores[i];

    /*An interesting score has been found*/
    if(current_score > 0.0){
      
      /*New start of a region*/
      if(start_position == -1){
	end_position = -1;
	start_position = i;
	
	local_max = current_score;
	index_local_max = i;

      }

      else{

	/*If the score is above the local maximum : continue*/
	if(current_score >= local_max){
	  local_max = current_score;
	  index_local_max = i;
	}

      }

    }
    /*Otherwise the score is 0 : whether it's an end of a region
     or we're not in an interesting region*/
    else{

      /*End of a region : new interval if its length is above the minimum
       length parameter and below max length*/
      if(start_position != -1){
	end_position = index_local_max;
	
	if(((end_position - start_position + 1) >= OPTS_min_length_region) &&
	   (end_position - start_position + 1) <= OPTS_max_length_region){

	  RENEW(m->interesting_regions->intervals,
		p_interval,
		m->interesting_regions->nb_intervals + 1);
	  NEW(m->interesting_regions->
	      intervals[m->interesting_regions->nb_intervals],
	      interval_s,
	      1);
	  m->interesting_regions->nb_intervals++;
	  m->interesting_regions->
	    intervals[m->interesting_regions->nb_intervals - 1]->pos_start = 
	    start_position;
	  m->interesting_regions->
	    intervals[m->interesting_regions->nb_intervals - 1]->pos_end = 
	    end_position;

	}

	local_max = -1;
	index_local_max = 0;
	start_position = -1;
	end_position = -1;
	 
      }

    }

  }

}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void determine_pos_query(p_matt_ygraph m, 
///                              int index_interval, 
///                              int index_al)
///
/// \brief Function used to determine the positions on the query sequence
/// for the conserved region on the reference
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
/// \param[in] index_interval : index of the reference interval in the array
/// of conserved region
/// \param[in] index_al : index of the alignment concerned 
///
/// That function will determine the positions on the query sequence for the 
/// the conserved region on the reference. It points at the correct interval
/// by the index_interval parameter and the correct alignment by the index_al
/// parameter.
///
/////////////////////////////////////////////////////////////////////////////

void determine_pos_query(p_matt_ygraph m, 
			 int index_interval, 
			 int index_al)
{

  /*Length of the interval on the reference sequence*/
  int length_interval_ref = 0;
  /*Length difference of the 2 starting positions*/
  int length_diff_start = 0;
  /*Length difference of the 2 ending positions*/
  int length_diff_end = 0;
  /*Max length of the query sequence : 50% gap max*/
  int max_length_query;

  /*Calculate the length of the interval and determine max length of the
   query sequence : 50%gap -> 2 * sizeofinterval */
  length_interval_ref = 
    m->interesting_regions->intervals[index_interval]->pos_end -
    m->interesting_regions->intervals[index_interval]->pos_start + 1;
  max_length_query = 2 * length_interval_ref;


  /*Calculate the length differences of the starting and ending 
    positions between the ref interval and the alignment*/
  length_diff_start = 
    m->interesting_regions->intervals[index_interval]->pos_start -
    m->array_of_al_info->al_info[index_al]->pos_start_ref;
  length_diff_end = 
    m->interesting_regions->intervals[index_interval]->pos_end -
    m->array_of_al_info->al_info[index_al]->pos_end_ref;

  /*Case the interval on the ref is included inside the alignment :
   positions on query won't change*/
  if((length_diff_start >= 0) && (length_diff_end <= 0)){

    m->sets_of_sequences->sets[index_interval]->
      q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
      pos_start = m->array_of_al_info->al_info[index_al]->pos_start_query;
    m->sets_of_sequences->sets[index_interval]->
      q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
      pos_end = m->array_of_al_info->al_info[index_al]->pos_end_query;
  }
  else{
    /*Case the alignment is included inside the ref interval :
     extend positions on query*/
    if((length_diff_start <= 0) && (length_diff_end >= 0)){
      
      m->sets_of_sequences->sets[index_interval]->
	q_intervals[m->sets_of_sequences->sets[index_interval]->
		    nb_q_intervals]->pos_start = 
	MAX(0,
	    m->array_of_al_info->al_info[index_al]->pos_start_query -
	    (2 * fabs(length_diff_start)));
      m->sets_of_sequences->sets[index_interval]->
	q_intervals[m->sets_of_sequences->sets[index_interval]->
		    nb_q_intervals]->pos_end = 
	MIN(m->array_of_seqs->seqs[m->array_of_al_info->al_info[index_al]->
				   index_query]->length - 1,
	    m->array_of_al_info->al_info[index_al]->pos_end_query +
	    (2 * length_diff_end));

    }
    /*Case the alignment and the interval only intersect*/
    else{
      
      /*Starting position of the interval is before the alignment :
       extend starting position on query*/
      if(length_diff_start <= 0){
	 m->sets_of_sequences->sets[index_interval]->
	q_intervals[m->sets_of_sequences->sets[index_interval]->
		    nb_q_intervals]->pos_start = 
	MAX(0,
	    m->array_of_al_info->al_info[index_al]->pos_start_query -
	    (2 * fabs(length_diff_start)));
	 m->sets_of_sequences->sets[index_interval]->
	   q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
	   pos_end = m->array_of_al_info->al_info[index_al]->pos_end_query; 

      }

      /*Last case : extending query on ending position*/
      else{
	m->sets_of_sequences->sets[index_interval]->
	  q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
	  pos_start = m->array_of_al_info->al_info[index_al]->pos_start_query;
	m->sets_of_sequences->sets[index_interval]->
	  q_intervals[m->sets_of_sequences->sets[index_interval]->
		      nb_q_intervals]->pos_end = 
	  MIN(m->array_of_seqs->seqs[m->array_of_al_info->al_info[index_al]->
				     index_query]->length - 1,
	      m->array_of_al_info->al_info[index_al]->pos_end_query +
	      (2 * length_diff_end));	
      }    
    }
  }


  /*Comment : the part after is not useful*/

  /*If the query size is above the max length : put -1 on starting position 
    to tell that this query is no valid*/
 /*  if(max_length_query < */
/*      (m->sets_of_sequences->sets[index_interval]-> */
/*       q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]-> */
/*       pos_end - */
/*       m->sets_of_sequences->sets[index_interval]-> */
/*       q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]-> */
/*       pos_start + 1)){ */
/*     m->sets_of_sequences->sets[index_interval]-> */
/*       q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]-> */
/*       pos_start = -1; */

/*   } */

}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void complete_set_sequences(p_matt_ygraph m)
///
/// \brief Function used to determine the sequences of the reference 
/// sequence and all query sequences of all sets
///
/// \param[in,out] m : the main structure containing all sets
///
/// That function will determine the sequences of the reference 
/// sequence and all query sequences according to positions and strand on all
/// sets of sequences
///
/////////////////////////////////////////////////////////////////////////////

void complete_set_sequences(p_matt_ygraph m)
{

  /*Indexes*/
  int i,j,k;
  /*Temporary start position*/
  int temp_start;
  /*Temporary end position*/
  int temp_end;
  /*Temporary query index*/
  int temp_index;


  /*Get all sets of sequences*/
  for(i = 0; i < m->sets_of_sequences->nb_sets; i++){

    temp_start = m->sets_of_sequences->sets[i]->ref_interval->pos_start;
    temp_end = m->sets_of_sequences->sets[i]->ref_interval->pos_end;

    /*Determine the reference sequence*/
    NEW(m->sets_of_sequences->sets[i]->ref_seq,
	char,
	temp_end - temp_start + 2);    
    
    /*Forward strand case*/
    if(m->sets_of_sequences->sets[i]->ref_interval->strand == '+'){
      
      for(j = temp_start; j <= temp_end; j++){
	m->sets_of_sequences->sets[i]->ref_seq[j - temp_start] = 
	  m->array_of_seqs->seqs[0]->bases[j];
      }
      m->sets_of_sequences->sets[i]->ref_seq[j - temp_start] = '\0';
    }
    /*Reverse strand case*/
    else{
      for(j = temp_end; j >= temp_start; j--){
	m->sets_of_sequences->sets[i]->ref_seq[temp_end - j] =
	  get_complementary_base(m->array_of_seqs->seqs[0]->bases[j]);
      }
      m->sets_of_sequences->sets[i]->ref_seq[temp_end - j] = '\0';
      
    }

    /*Determine all query sequences of the set*/
    for(k = 0; k < m->sets_of_sequences->sets[i]->nb_q_intervals; k++){

      
      temp_start = m->sets_of_sequences->sets[i]->q_intervals[k]->pos_start;
      temp_end = m->sets_of_sequences->sets[i]->q_intervals[k]->pos_end;
      temp_index = m->sets_of_sequences->sets[i]->q_intervals[k]->index;

      NEW(m->sets_of_sequences->sets[i]->q_intervals[k]->seq,
	  char,
	  temp_end - temp_start + 2);
      
      /*Forward strand case*/
      if(m->sets_of_sequences->sets[i]->q_intervals[k]->strand == '+'){	

	for(j = temp_start; j <= temp_end; j++){
	  m->sets_of_sequences->sets[i]->q_intervals[k]->seq[j - temp_start] = 
	    m->array_of_seqs->seqs[temp_index]->bases[j];
	}
	m->sets_of_sequences->sets[i]->q_intervals[k]->seq[j - temp_start] = '\0';

      }
      /*Reverse strand case*/
      else{

	for(j = temp_end; j >= temp_start; j--){

	  m->sets_of_sequences->sets[i]->q_intervals[k]->seq[temp_end - j] =
	    get_complementary_base(m->array_of_seqs->seqs[temp_index]->bases[j]);

	}

	m->sets_of_sequences->sets[i]->q_intervals[k]->seq[temp_end - j] = '\0';
	
      }
           
    } 
  }
}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void get_sets_of_sequences(p_matt_ygraph m)
///
/// \brief Function used to determine the sets of sequences of conserved 
/// regions
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated with the sets
///
/// That function will determine the positions on query sequences according
/// to the positions of conserved regions on the reference and the info
/// about alignments
///
/////////////////////////////////////////////////////////////////////////////


void get_sets_of_sequences(p_matt_ygraph m)
{

  /*Indexes*/
  int i,j;  
  /*Boolean that says if the current interval has been compared
   to all possible intervals in the array of alignment*/
  int comparison_done = 0;
  /*Intersect percent*/
  float percent_intersect = 0.2;
  /*Length of the intersection*/
  int length_intersect;


  /*Initialize the array of sets of sequence : the number of regions*/
  NEW(m->sets_of_sequences->sets,
      p_set_of_seqs, 
      m->interesting_regions->nb_intervals);
  m->sets_of_sequences->nb_sets = m->interesting_regions->nb_intervals;

  /*Get all regions on the reference and search for sequences
   that intersect according to the array of alignments*/  
  for(i = 0; i < m->interesting_regions->nb_intervals; i++){

    comparison_done = 0;
    NEW(m->sets_of_sequences->sets[i],
	set_of_seqs_s,
	1);
    NEW(m->sets_of_sequences->sets[i]->ref_interval,
	interval_s,
	1);
    m->sets_of_sequences->sets[i]->q_intervals = NULL;
    m->sets_of_sequences->sets[i]->nb_q_intervals = 0;
    m->sets_of_sequences->sets[i]->ref_interval->pos_start =
      m->interesting_regions->intervals[i]->pos_start;
    m->sets_of_sequences->sets[i]->ref_interval->pos_end =
      m->interesting_regions->intervals[i]->pos_end;


    /*Search the alignments that intersects the current interval until
     comparison is done*/
    for(j = 0;
	(!comparison_done) &&
	  (j < m->array_of_al_info->nb_al_info);
	j++){


      /*Since the array of alignment info has been sorted : check if
       all the possible comparison has been done*/
      if(m->array_of_al_info->al_info[j]->pos_start_ref >
	 m->interesting_regions->intervals[i]->pos_end){
	comparison_done = 1;
      }
      else{
	/*Check if the current alignment intersects the actual one*/
	if(is_ref_intersect(m->array_of_al_info->al_info[j],
			    m->interesting_regions->intervals[i])){
	  

	  /*Get the length of the intersection on the reference*/
	  length_intersect = 
	    get_length_ref_intersect(m->array_of_al_info->al_info[j],
				     m->interesting_regions->intervals[i]);

	  /*If the length if enough big according to the percentage
	   Add the needed information in the sets of sequences*/
	  if(length_intersect >= 
	     (percent_intersect *
	      (m->interesting_regions->intervals[i]->pos_end -
	       m->interesting_regions->intervals[i]->pos_start + 1))){
	    

	    RENEW(m->sets_of_sequences->sets[i]->q_intervals,
		  p_interval_query,
		  m->sets_of_sequences->sets[i]->nb_q_intervals
		  + 1);
	    NEW(m->sets_of_sequences->sets[i]->
		q_intervals[m->sets_of_sequences->sets[i]->nb_q_intervals],
		interval_query_s,
		1);
	    /*Determine the positions on the query sequence*/
	    determine_pos_query(m, i, j);

	    /*If starting position is -1 : the query is no valid*/
	    if(m->sets_of_sequences->sets[i]->
	       q_intervals[m->sets_of_sequences->sets[i]->nb_q_intervals]->
	       pos_start == -1){

	      DESTROY(m->sets_of_sequences->sets[i]->
		      q_intervals[m->sets_of_sequences->sets[i]->
				  nb_q_intervals]);
	      
	    }
	    /*Add the index and strands attribute*/
	    else{
	      m->sets_of_sequences->sets[i]->ref_interval->strand =
		m->array_of_al_info->al_info[j]->strand_ref; 

	      m->sets_of_sequences->sets[i]->
		q_intervals[m->sets_of_sequences->sets[i]->nb_q_intervals]->
		index = m->array_of_al_info->al_info[j]->index_query;

	      m->sets_of_sequences->sets[i]->
		q_intervals[m->sets_of_sequences->sets[i]->nb_q_intervals]->
		strand = m->array_of_al_info->al_info[j]->strand_query;
	      
	      
	      m->sets_of_sequences->sets[i]->nb_q_intervals++;

	    } 
	  }
	}	
      }      
    }
 
 
  }

  

  /*Get reference sequence and all query sequences on all found sets*/
  complete_set_sequences(m);

}

////////////////////////////////////////////////////////////////////////////
///
/// \fn void eliminate_redundancy_set(p_matt_ygraph m, int index_set)
///
/// \brief Function used to eliminate redundant sequences inside the 
/// current set
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
/// \param[in] index_set : the index of the set to process
///
/// That function will eliminate redundant sequences inside the set (same
/// species and overlapped positions) 
///
/////////////////////////////////////////////////////////////////////////////


void eliminate_redundancy_set(p_matt_ygraph m, int index_set)
{

  /*Indexes*/
  int i,j,k;
  /*Species of the current sequence*/
  char *current_species = NULL;
  /*Index of the current sequence*/
  int current_index = 0;
  /*Species of the compared sequence*/
  char *compared_species = NULL;
  /*Index of the compared sequence*/
  int compared_index = 0;
  /*Array of indexes to destroy*/
  int *to_destroy = NULL;
  /*Number of sequences to destroy*/
  int nb_to_destroy = 0;

  /*Process all queries of the set*/
  for(i = 0; i < m->sets_of_sequences->sets[index_set]->nb_q_intervals; i++){

    /*Get species*/
    current_index = m->sets_of_sequences->sets[index_set]->q_intervals[i]->index;
    NEW(current_species,
	char,
	strlen(m->array_of_seqs->seqs[current_index]->species) + 1);
    strcpy(current_species,
	   m->array_of_seqs->seqs[current_index]->species);
    

    /*Check all queries that has the same species and overlap positions*/
    for(j = i + 1; j < m->sets_of_sequences->sets[index_set]->nb_q_intervals;
	j++){

      compared_index = m->sets_of_sequences->sets[index_set]->q_intervals[j]->index;
      NEW(compared_species,
	  char,
	  strlen(m->array_of_seqs->seqs[compared_index]->species) + 1);
      strcpy(compared_species,
	     m->array_of_seqs->seqs[compared_index]->species);

      if(strcmp(current_species,compared_species) == 0){

	/*Case identical queries or query "j" include inside query "i"*/
	if((m->sets_of_sequences->sets[index_set]->q_intervals[i]->pos_start <=
	    m->sets_of_sequences->sets[index_set]->q_intervals[j]->pos_start) &&
	   (m->sets_of_sequences->sets[index_set]->q_intervals[i]->pos_end >=
	    m->sets_of_sequences->sets[index_set]->q_intervals[j]->pos_end)){

	  /*Destroy the redundant sequence*/
	  DESTROY(m->sets_of_sequences->sets[index_set]->q_intervals[j]->seq);
	  DESTROY(m->sets_of_sequences->sets[index_set]->q_intervals[j]);

	  for(k = j;
	      k < m->sets_of_sequences->sets[index_set]->nb_q_intervals - 1;
	      k++){

	  NEW(m->sets_of_sequences->sets[index_set]->q_intervals[k],
	      interval_query_s,
	      1);
	  
	  copy_interval_query(m->sets_of_sequences->sets[index_set]->q_intervals[k],
			      m->sets_of_sequences->sets[index_set]->q_intervals[k + 1]);

	  DESTROY(m->sets_of_sequences->sets[index_set]->q_intervals[k + 1]->seq);
	  DESTROY(m->sets_of_sequences->sets[index_set]->q_intervals[k + 1]);

	  }
	  m->sets_of_sequences->sets[index_set]->nb_q_intervals--;
	  j--;

	}
	else{
	  
	  /*query "i" include inside query "j"*/
	  if((m->sets_of_sequences->sets[index_set]->q_intervals[i]->pos_start >=
	      m->sets_of_sequences->sets[index_set]->q_intervals[j]->pos_start) &&
	     (m->sets_of_sequences->sets[index_set]->q_intervals[i]->pos_end <=
	      m->sets_of_sequences->sets[index_set]->q_intervals[j]->pos_end)){

	    if((nb_to_destroy == 0) ||
	       !is_present_int_array(i,to_destroy,nb_to_destroy)){
	      RENEW(to_destroy,
		    int,
		    nb_to_destroy + 1);
	      to_destroy[nb_to_destroy] = i;
	      nb_to_destroy++;
	    }
	  }


	}

      }
      
      
      DESTROY(compared_species);

    }

    

    
    DESTROY(current_species);

  }

  /*Destroy queries*/
  for(i = 0; i < nb_to_destroy; i++){

    current_index = to_destroy[i];

     DESTROY(m->sets_of_sequences->sets[index_set]->q_intervals[current_index - i]->seq);
     DESTROY(m->sets_of_sequences->sets[index_set]->q_intervals[current_index - i]);
      
      for(k = current_index - i;
	  k < m->sets_of_sequences->sets[index_set]->nb_q_intervals - 1 - i;
	  k++){
	NEW(m->sets_of_sequences->sets[index_set]->q_intervals[k],
	    interval_query_s,
	    1);

	copy_interval_query(m->sets_of_sequences->sets[index_set]->q_intervals[k],
			    m->sets_of_sequences->sets[index_set]->q_intervals[k + 1]);
	
	DESTROY(m->sets_of_sequences->sets[index_set]->q_intervals[k + 1]->seq);
	DESTROY(m->sets_of_sequences->sets[index_set]->q_intervals[k + 1]);
	
      }

  
  }
  m->sets_of_sequences->sets[index_set]->nb_q_intervals -= nb_to_destroy;
  DESTROY(to_destroy);

}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void compare_all_against_ref(p_matt_ygraph m)
///
/// \brief Function used to compare all sequences against the reference
/// for all sets of sequences
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
///
/// That function will compare all sequences against the reference
/// for all sets of sequences and eliminate all queries that don't repsect
/// the identity thresholds (min and max)
///
/////////////////////////////////////////////////////////////////////////////


void compare_all_against_ref(p_matt_ygraph m)
{

  /*Indexes*/
  int i,j,k;
  /*Array of indexes of sequences in the current set to destroy*/
  int *to_destroy = NULL;
  /*Number of sequences to destroy*/
  int nb_to_destroy = 0;
  /*Current indes of the sequence to destroy*/
  int current_index = 0;
  /*Array of sequences for the carnac call*/
  p_sequence *seqs_carnac = NULL;
  /*Boolean that says if a sequence is good according to the identity
   thresholds and/or carnac*/
  int is_seq_good = 0;


  /*Get all sets of sequences*/
  for(i = 0; i < m->sets_of_sequences->nb_sets; i++){
  
    if(to_destroy != NULL){
      DESTROY(to_destroy);
      nb_to_destroy = 0;
    }

    if(OPTS_verbose){
      print_verbose_start_compare_ref_set(i + 1,
					  m->sets_of_sequences->
					  sets[i]->nb_q_intervals + 1);

    }

    /*Create an array of sequence with the reference sequence if
      carnac is wanted*/
    if(OPTS_carnac){

      if(seqs_carnac != NULL){
	
	if(seqs_carnac[1] != NULL){
	  DESTROY(seqs_carnac[1]->name);
	  DESTROY(seqs_carnac[1]->bases);
	  DESTROY(seqs_carnac[1]);

	}

	if(seqs_carnac[0] != NULL){
	  DESTROY(seqs_carnac[0]->name);
	  DESTROY(seqs_carnac[0]->bases);
	  DESTROY(seqs_carnac[0]);

	}    

	DESTROY(seqs_carnac);

      }

      
      NEW(seqs_carnac, p_sequence, 2);

      /*First sequence : the reference*/
      NEW(seqs_carnac[0], sequence_s, 1);
      NEW(seqs_carnac[0]->bases,
	  char,
	  strlen(m->sets_of_sequences->sets[i]->ref_seq) + 1);
      NEW(seqs_carnac[0]->name,
	  char,
	  4);
      strcpy(seqs_carnac[0]->bases,
	     m->sets_of_sequences->sets[i]->ref_seq);
      strcpy(seqs_carnac[0]->name, "ref");
      seqs_carnac[0]->length = 
	strlen(m->sets_of_sequences->sets[i]->ref_seq);

    }


    /*Get all queries and compare them with the reference*/
    for(j = 0; j < m->sets_of_sequences->sets[i]->nb_q_intervals; j++){

      is_seq_good = 1;

      small_in_large(m->sets_of_sequences->sets[i]->ref_seq,
		     m->sets_of_sequences->sets[i]->q_intervals[j]);

 
      /*Check identity constraints*/
      if((m->sets_of_sequences->sets[i]->q_intervals[j]->id_ref < 
	  OPTS_min_id) ||
	 (m->sets_of_sequences->sets[i]->q_intervals[j]->id_ref > 
	  OPTS_max_id)){
	
	is_seq_good = 0;
      }
      else{

	/*Check carnac if wanted*/
	if(OPTS_carnac){

	  if(seqs_carnac[1] != NULL){

	    DESTROY(seqs_carnac[1]->name);
	    DESTROY(seqs_carnac[1]->bases);
	    DESTROY(seqs_carnac[1]);
	      
	  }	  

	  /*Create a temporary sequence for the current query*/
	  NEW(seqs_carnac[1], sequence_s, 1);
	  NEW(seqs_carnac[1]->bases, 
	      char, 
	      strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq) + 1);
	  NEW(seqs_carnac[1]->name,
	      char,
	      5);
	  strcpy(seqs_carnac[1]->bases, 
		 m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
	  strcpy(seqs_carnac[1]->name, "temp");
	  seqs_carnac[1]->length = 
	    strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq);

	  
	  if(!call_carnac(seqs_carnac, 2)){
	    is_seq_good = 0;
	  }


	}
      }


      /*Destroy sequences that is not good*/

      if(!is_seq_good){

	RENEW(to_destroy, int, nb_to_destroy + 1);
	to_destroy[nb_to_destroy] = j;
	nb_to_destroy++;
	
      }
    }

    /*Destroy sequences that don't verify identity constraints with the 
      reference*/
    for(j = 0; j < nb_to_destroy; j++){
      
      current_index = to_destroy[j];

      DESTROY(m->sets_of_sequences->sets[i]->q_intervals[current_index - j]->seq);
      DESTROY(m->sets_of_sequences->sets[i]->q_intervals[current_index - j]);
      
      for(k = current_index - j;
	  k < m->sets_of_sequences->sets[i]->nb_q_intervals - 1 - j;
	  k++){
	NEW(m->sets_of_sequences->sets[i]->q_intervals[k],
	    interval_query_s,
	    1);

	copy_interval_query(m->sets_of_sequences->sets[i]->q_intervals[k],
			    m->sets_of_sequences->sets[i]->q_intervals[k + 1]);
	
	DESTROY(m->sets_of_sequences->sets[i]->q_intervals[k + 1]->seq);
	DESTROY(m->sets_of_sequences->sets[i]->q_intervals[k + 1]);
	
      }

    }

    m->sets_of_sequences->sets[i]->nb_q_intervals -= nb_to_destroy;
    if(m->sets_of_sequences->sets[i]->nb_q_intervals != 0){
      RENEW(m->sets_of_sequences->sets[i]->q_intervals,
	    p_interval_query,
	    m->sets_of_sequences->sets[i]->nb_q_intervals);
    }
    else{
      DESTROY(m->sets_of_sequences->sets[i]->q_intervals);
    }


    /*Erase redundant sequences inside the set*/
    eliminate_redundancy_set(m,i);


    if(OPTS_verbose){
      print_verbose_end_compare_ref_set(m->sets_of_sequences->
					sets[i]->nb_q_intervals + 1);
    }


  }

  if(to_destroy != NULL){
    DESTROY(to_destroy);
  }

  if(OPTS_carnac){

    if(seqs_carnac != NULL){
      
      if(seqs_carnac[1] != NULL){
	DESTROY(seqs_carnac[1]->name);
	DESTROY(seqs_carnac[1]->bases);
	DESTROY(seqs_carnac[1]);
	
      }
      
      if(seqs_carnac[0] != NULL){
	DESTROY(seqs_carnac[0]->name);
	DESTROY(seqs_carnac[0]->bases);
	DESTROY(seqs_carnac[0]);
	
	}    
      
      DESTROY(seqs_carnac);
      
    }
  }
    
}

/////////////////////////////////////////////////////////////////////////////
///
/// \fn int compare_sort_id(const void *a, const void *b)
///
/// \brief Function used to compare two p_interval_query by their ref 
/// identity
///
/// \param[in] a :first query to compare
/// \param[in] b : second query to compare
///
/// \return -1 if "a" has a lower id ref than "b", 0 if equal and -1
/// otherwise
///
/// Function used to compare two p_interval_query by their ref 
/// identity
///
///
/////////////////////////////////////////////////////////////////////////////

int compare_sort_id(const void *a, const void *b)
{
  /*Args retrievment*/
  p_interval_query *arg1 = (p_interval_query *) a;
  p_interval_query *arg2 = (p_interval_query *) b;
  /*Position comparison*/
  if((*arg1)->id_ref < (*arg2)->id_ref) return 1;
  else if ((*arg1)->id_ref == (*arg2)->id_ref) return 0;
  else return -1;
}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void sort_identity_ref(p_matt_ygraph m)
///
/// \brief Function used to sort all queries in each set by reference 
/// identity
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
///
/// Function used to sort all queries in each set by reference 
/// identity
///
/////////////////////////////////////////////////////////////////////////////


void sort_identity_ref(p_matt_ygraph m)
{

  /*Index*/
  int i;

  /*Get all sets of sequences*/
  for(i = 0; i < m->sets_of_sequences->nb_sets; i++){ 

    /*Sort all queries*/
    qsort(m->sets_of_sequences->sets[i]->q_intervals,
	  m->sets_of_sequences->sets[i]->nb_q_intervals,
	  sizeof(p_interval_query),
	  compare_sort_id);
 
  }
  
}

////////////////////////////////////////////////////////////////////////////
///
/// \fn void check_species_repetitions(p_matt_ygraph m)
///
/// \brief Function used to check species repetitions
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
///
/// That function will check if the species repetitions threshold is
/// respected
///
/////////////////////////////////////////////////////////////////////////////

void check_species_repetitions(p_matt_ygraph m)
{

  /*Indexes*/
  int i,j,k;
  /*Array of processed species*/
  char_array *processed_species = NULL;
  /*Index of the current sequence*/
  int current_index = 0;
  /*Species of the current sequence*/
  char *current_species = NULL;
  /*Index of the compared sequence*/
  int compared_index = 0;
  /*Species of the current compared sequence*/
  char *compared_species = NULL;
  /*Number of the current species*/
  int cpt_species = 0;
  /*Boolean that says if a species is good according to the max number
   of repetitions*/
  int is_species_good = 1;


  /*Initialize the array of processed species*/
  NEW(processed_species, char_array, 1);
  processed_species->chars = NULL;
  processed_species->nb_chars = 0;

  /*Get all sets of sequences*/
  for(i = 0; i < m->sets_of_sequences->nb_sets; i++){
    if(OPTS_verbose){
      print_verbose_start_checking_repetitions_set(i + 1, 
					   m->sets_of_sequences->sets[i]->
					   nb_q_intervals + 1);
    }

    /*The set contains enough sequences to check the species*/
    if(m->sets_of_sequences->sets[i]->nb_q_intervals > OPTS_max_species_repet){

      if(processed_species->chars){
	for(j = 0; j < processed_species->nb_chars; j++){
	  DESTROY(processed_species->chars[j]);
	}
	DESTROY(processed_species->chars);
	processed_species->nb_chars = 0;
	
      }
      processed_species->chars = NULL;
      processed_species->nb_chars = 0;
      is_species_good = 1;

      /*Check all query species and stop if one doesn't respect the
       max number of species*/
      for(j = 0; is_species_good && 
	    j < m->sets_of_sequences->sets[i]->nb_q_intervals; j++){
	current_index = m->sets_of_sequences->sets[i]->q_intervals[j]->index;
	current_species = m->array_of_seqs->seqs[current_index]->species;

	cpt_species = 0;

	/*Check if the species has not been already processed*/
	if(!is_present_char_array(current_species,processed_species)){
	  
	  /*Count the number of species*/
	  for(k = j + 1; is_species_good &&
		k < m->sets_of_sequences->sets[i]->nb_q_intervals; k++){
	    
	    compared_index = 
	      m->sets_of_sequences->sets[i]->q_intervals[k]->index;
	    compared_species = m->array_of_seqs->seqs[compared_index]->species;

	    if(strcmp(compared_species,current_species) == 0){
	      cpt_species++;

	      /*If the maximum is reached stop*/
	      if(cpt_species > OPTS_max_species_repet){
		is_species_good = 0;
	      }

	    }
	      

	  }

	  /*Add the current species to the processed ones*/
	  if(is_species_good){
	    RENEW(processed_species->chars, 
		  char*, 
		  processed_species->nb_chars + 1);
	    NEW(processed_species->chars[processed_species->nb_chars],
		char,
		strlen(current_species) + 1);
	    strcpy(processed_species->chars[processed_species->nb_chars], 
		   current_species);
	    processed_species->nb_chars++;
	  }

	}

      }

      /*If the species is not good : destroy the queries to tell
       that this set is not good*/
      if(!is_species_good){
	for(k = 0; k < m->sets_of_sequences->sets[i]->nb_q_intervals; k++){
	  if(m->sets_of_sequences->sets[i]->q_intervals[k]->seq){
	    DESTROY(m->sets_of_sequences->sets[i]->q_intervals[k]->seq);
	  }
	  DESTROY(m->sets_of_sequences->sets[i]->q_intervals[k]);
	}

	DESTROY(m->sets_of_sequences->sets[i]->q_intervals);
	m->sets_of_sequences->sets[i]->nb_q_intervals = 0;
      }
    }


    if(OPTS_verbose){

      print_verbose_end_checking_repetitions_set(m->sets_of_sequences->
						 sets[i]->nb_q_intervals + 1);

    }
  }

  if(processed_species){
    if(processed_species->chars){
      for(j = 0; j < processed_species->nb_chars; j++){
	DESTROY(processed_species->chars[j]);
      }
      DESTROY(processed_species->chars);
    }
    DESTROY(processed_species);
  }



}



////////////////////////////////////////////////////////////////////////////
///
/// \fn void search_naive_clique(p_matt_ygraph m)
///
/// \brief Function used to serach the naive clique for all sets
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
///
/// That function will search the naive clique for all sets. It will include
/// by reference identity order each sequences and abort as soon as one
/// cannot be included
///
/////////////////////////////////////////////////////////////////////////////


void search_naive_clique(p_matt_ygraph m)
{

  /*Indexes*/
  int i,j,k;
  /*Number of sequences wanted*/
  int nb_seqs_wanted = 0;
  /*Boolean that says if the clique is good*/
  int is_clique_good = 1;
  /*Current identity*/
  float identity = 0.0;
  /*Current index of the clique sequence in the query array*/
  int index_current = -1;
  /*Number of sequences in the current clique*/
  int nb_seqs_clique = 0;
  /*Array of sequences for the carnac call*/
  p_sequence *seqs_carnac = NULL;


  /*Manage number of sequences by set wanted (ref + queries !!)*/
  if(OPTS_nb_seqs_set == -1){
    nb_seqs_wanted = 100000;
  }
  else{
    nb_seqs_wanted = OPTS_nb_seqs_set;
  }


  /*Get all sets of sequences*/
   for(i = 0; i < m->sets_of_sequences->nb_sets; i++){ 

    /*Initialize the clique structure*/
    NEW(m->sets_of_sequences->sets[i]->clique_set,
	clique_s,
	1);

    m->sets_of_sequences->sets[i]->clique_set->clique_index = NULL;
    m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique = 0;
    
    nb_seqs_clique = 0;
    is_clique_good = 1;
    
    /*Process only sets which have at least one query sequence*/
    if(m->sets_of_sequences->sets[i]->nb_q_intervals > 0){

      if(OPTS_verbose){
      print_verbose_start_naive_clique_set(i + 1, 
					   m->sets_of_sequences->sets[i]->
					   nb_q_intervals + 1);
      }

      /*Adding the first query sequence in the clique : it respects the 
	identity threshold with the reference sequence*/
      RENEW(m->sets_of_sequences->sets[i]->clique_set->clique_index,
	    int,
	    1);
      m->sets_of_sequences->sets[i]->clique_set->clique_index[0] = 0;
      nb_seqs_clique = 1;    

      
      /*Get all query sequences*/
      for(j = 1; 
	  is_clique_good && 
	    nb_seqs_clique < (nb_seqs_wanted - 1) &&
	    j < m->sets_of_sequences->sets[i]->nb_q_intervals;
	  j++){
	

	/*Compare the current query with all seqs already inside the clique
	  until the number of sequences wanted is reached or until a 
	  sequence can't be integrated inside the clique*/
	for(k = 0;
	    is_clique_good &&
	      k < nb_seqs_clique;
	    k++){
	
	  index_current = 
	    m->sets_of_sequences->sets[i]->clique_set->clique_index[k];

	  identity = 
	    aln(m->sets_of_sequences->sets[i]->q_intervals[j],
		m->sets_of_sequences->sets[i]->q_intervals[index_current]);


	  if((identity < OPTS_min_id) || (identity > OPTS_max_id)){
	    is_clique_good = 0;
	  }
	  /*Check carnac if wanted*/
	  else{

	    if(OPTS_carnac){
	      NEW(seqs_carnac, p_sequence, 2);
	      NEW(seqs_carnac[0], sequence_s, 1);
	      NEW(seqs_carnac[0]->name, char, 5);
	      NEW(seqs_carnac[0]->bases, 
		  char,
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq) + 1);
	      strcpy(seqs_carnac[0]->name, "seq1");
	      strcpy(seqs_carnac[0]->bases,
		     m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
	      seqs_carnac[0]->length = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq);

	      NEW(seqs_carnac[1], sequence_s, 1);
	      NEW(seqs_carnac[1]->name, char, 5);
	      NEW(seqs_carnac[1]->bases, 
		  char,
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[index_current]->seq) + 1);
	      strcpy(seqs_carnac[1]->name, "seq2");
	      strcpy(seqs_carnac[1]->bases,
		     m->sets_of_sequences->sets[i]->q_intervals[index_current]->seq);
	      seqs_carnac[1]->length = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[index_current]->seq);


	      if(!call_carnac(seqs_carnac, 2)){
		is_clique_good = 0;
	      }

	      DESTROY(seqs_carnac[0]->name);
	      DESTROY(seqs_carnac[0]->bases);
	      DESTROY(seqs_carnac[1]->name);
	      DESTROY(seqs_carnac[1]->bases);
	      DESTROY(seqs_carnac[0]);
	      DESTROY(seqs_carnac[1]);
	      DESTROY(seqs_carnac);

	    }

	  }

	}

	if(is_clique_good){
	  RENEW(m->sets_of_sequences->sets[i]->clique_set->clique_index,
		int,
		nb_seqs_clique + 1);
	  m->sets_of_sequences->sets[i]->clique_set->clique_index[nb_seqs_clique] =
	    j;
	  nb_seqs_clique++;
	    
	}

      }

      if(is_clique_good){
	m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique = 
	  nb_seqs_clique;
	if(OPTS_verbose){    
	  print_verbose_end_naive_clique_set(nb_seqs_clique + 1);
	}
      }
      else{
	DESTROY(m->sets_of_sequences->sets[i]->clique_set->clique_index);
	DESTROY(m->sets_of_sequences->sets[i]->clique_set);
	print_verbose_end_naive_clique_set(0);

      }

    }
    else{
      if(OPTS_verbose){
	print_verbose_no_connect(i + 1);
      }
    }

  }
    
  
}


////////////////////////////////////////////////////////////////////////////
///
/// \fn void recursive_search_maxi_clique(p_set_of_seqs set,
///                                       p_clique temp_clique,
///                                       int index)
///
/// \brief Function used to search recursively the maxi clique in the
/// current set
///
/// \param[in,out] set : the current set
/// \param[in,out] temp_clique : the temporary maxi clique
/// \param[in] index : the index in the query array of the sequence to add 
///
/// That function will search the maxi clique recursively by trying to 
/// add the query sequence in the temporary maxi clique
///
/////////////////////////////////////////////////////////////////////////////

void recursive_search_maxi_clique(p_set_of_seqs set,
				  p_clique temp_clique,
				  int index)
{

  /*Index*/
  int i;
  /*Boolean that says if the current sequence is connected with every
    sequence in the clique*/
  int is_connected = 1;
  /*Temp index of a sequence*/
  int temp_index;
  /*Last length of the temporary max clique*/
  int last_length = 0;
  /*Boolean that says if the maxi clique has been found*/
  int is_finished = 0;

  if(((OPTS_nb_seqs_set != -1) && 
      ( set->clique_set->nb_seqs_clique < OPTS_nb_seqs_set - 1))||
     ((OPTS_nb_seqs_set == -1) &&
      (set->clique_set->nb_seqs_clique !=
       set->nb_q_intervals))){


    /*Check if the sequence to add in not already in the clique*/
    if(!is_present_int_array(index,
			     temp_clique->clique_index,
			     temp_clique->nb_seqs_clique)){

      /*Check if the sequence is connected with every sequences of the clique
	except the last one :  it's obliged to be a neighbor of the current one*/
      for(i = 0; is_connected && i < temp_clique->nb_seqs_clique - 1; i++){
      
	temp_index = temp_clique->clique_index[i];
	if(!is_present_int_array(index,
				 set->neighbors[temp_index]->indexes,
				 set->neighbors[temp_index]->nb_neighbors)){
	  is_connected = 0;
	}
      }

      /*If the sequence is connected with every sequences of the clique,
	add it to the temporary clique ans try to add its neighbors*/
      if(is_connected){

	RENEW(temp_clique->clique_index,
	      int,
	      temp_clique->nb_seqs_clique + 1);
	temp_clique->clique_index[temp_clique->nb_seqs_clique] = index;
	temp_clique->nb_seqs_clique++;
      
	/*If the size of the temporary clique is greater than the current
	  one in the set : copy the temp clique. The size of the clique must 
	  respect the max number of sequences wanted by the user*/
	if(temp_clique->nb_seqs_clique > set->clique_set->nb_seqs_clique){


	  DESTROY(set->clique_set->clique_index);
	  NEW(set->clique_set->clique_index,
	      int,
	      temp_clique->nb_seqs_clique);
	  for(i = 0; i < temp_clique->nb_seqs_clique; i++){
	    set->clique_set->clique_index[i] = temp_clique->clique_index[i];
	  }
	  set->clique_set->nb_seqs_clique = temp_clique->nb_seqs_clique;
	}

      
	if(((OPTS_nb_seqs_set != -1) &&
	    ( set->clique_set->nb_seqs_clique < OPTS_nb_seqs_set - 1))||
	   ((OPTS_nb_seqs_set == -1) &&
	   (set->clique_set->nb_seqs_clique !=
	    set->nb_q_intervals))){	
	  for(i = 0; !is_finished && 
		i < set->neighbors[index]->nb_neighbors; i++){
	
	    /*Search recursively with all neighbors of sequence that has just
	      been added if the max size has not been reached*/
	    if(((OPTS_nb_seqs_set != -1) && 
		( set->clique_set->nb_seqs_clique < OPTS_nb_seqs_set - 1))||
	       ((OPTS_nb_seqs_set == -1) &&
		(set->clique_set->nb_seqs_clique !=
		 set->nb_q_intervals))){
	  
	  
	      temp_index = set->neighbors[index]->indexes[i];
	  
	      last_length = temp_clique->nb_seqs_clique;
	  
	      recursive_search_maxi_clique(set,
					   temp_clique,
					   temp_index);
	  
	      /*If the temporary clique has been modified erase the last
		index added and continue the recursion*/
	      if(last_length != temp_clique->nb_seqs_clique){
		RENEW(temp_clique->clique_index, 
		      int,
		      temp_clique->nb_seqs_clique - 1);
		temp_clique->nb_seqs_clique--;
		
	      }
	      
	      
	    }
	    else{
	      is_finished = 1;
	    }
	  }
	}
	else{
	  is_finished = 1;
	}    
      }
    }
  }
}




////////////////////////////////////////////////////////////////////////////
///
/// \fn void maxi_clique_search(p_matt_ygraph m)
///
/// \brief Function used to search the maxi clique for all sets
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
///
/// That function will search the maxi clique for all sets. If a
/// number of sequences has been provided by the user the maxi clique will
/// be at least that size
///
/////////////////////////////////////////////////////////////////////////////

void maxi_clique_search(p_matt_ygraph m)
{

  /*Indexes*/
  int i,j,k;
  /*Current identity*/
  float current_id;
  /*Current number of neighbors for the current query*/
  int nb_neighbors = 0;
  /*Temporary clique*/
  p_clique temp_clique;
  /*Boolean that says if the search is finished*/
  int is_finished = 0;
  /*Array of sequences for the carnac call*/
  p_sequence *seqs_carnac = NULL;
  /*Boolean that says if a sequence is connected with the current other
   one*/
  int is_connected = 0;


  /*Get all sets*/
  for(i = 0; i < m->sets_of_sequences->nb_sets; i++){
    /*Process only sets which have at least one query sequence*/
    if(m->sets_of_sequences->sets[i]->nb_q_intervals > 0){

      is_connected = 1;

      /*Intialize and update the neighbor array according to identity
	thresholds by doing every alignments in the set*/
      NEW(m->sets_of_sequences->sets[i]->neighbors,
	  p_neighbor,
	  m->sets_of_sequences->sets[i]->nb_q_intervals);
      
      if(OPTS_verbose){
	print_verbose_start_maxi_clique_set(i + 1, 
					    m->sets_of_sequences->sets[i]->
					    nb_q_intervals + 1);
      }

      for(j = 0; j < m->sets_of_sequences->sets[i]->nb_q_intervals; j++){

	NEW(m->sets_of_sequences->sets[i]->neighbors[j],
	    neighbor_s,
	    1);
	m->sets_of_sequences->sets[i]->neighbors[j]->indexes = NULL;
	m->sets_of_sequences->sets[i]->neighbors[j]->nb_neighbors = 0;
	nb_neighbors = 0;

	for(k = j + 1; k < m->sets_of_sequences->sets[i]->nb_q_intervals; k++){

	  current_id = aln(m->sets_of_sequences->sets[i]->q_intervals[j],
			   m->sets_of_sequences->sets[i]->q_intervals[k]);


	  /*The identity respects the identity thresholds : add the neighbor*/
	  if((current_id < OPTS_min_id) || (current_id > OPTS_max_id)){

	    is_connected = 0;

	  }

	  if(is_connected){

	    /*Check carnac if wanted*/
	    if(OPTS_carnac){
	    
	      NEW(seqs_carnac, p_sequence, 2);
	      NEW(seqs_carnac[0], sequence_s, 1);
	      NEW(seqs_carnac[0]->name, char, 5);
	      NEW(seqs_carnac[0]->bases, 
		  char,
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq) + 1);
	      strcpy(seqs_carnac[0]->name, "seq1");
	      strcpy(seqs_carnac[0]->bases,
		     m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
	      seqs_carnac[0]->length = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq);

	      NEW(seqs_carnac[1], sequence_s, 1);
	      NEW(seqs_carnac[1]->name, char, 5);
	      NEW(seqs_carnac[1]->bases, 
		  char,
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[k]->seq) + 1);
	      strcpy(seqs_carnac[1]->name, "seq2");
	      strcpy(seqs_carnac[1]->bases,
		     m->sets_of_sequences->sets[i]->q_intervals[k]->seq);
	      seqs_carnac[1]->length = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[k]->seq);


	      if(!call_carnac(seqs_carnac, 2)){
		is_connected = 0;
	      }

	      DESTROY(seqs_carnac[0]->name);
	      DESTROY(seqs_carnac[0]->bases);
	      DESTROY(seqs_carnac[1]->name);
	      DESTROY(seqs_carnac[1]->bases);
	      DESTROY(seqs_carnac[0]);
	      DESTROY(seqs_carnac[1]);
	      DESTROY(seqs_carnac);



	    }

	    if(is_connected){

	      RENEW(m->sets_of_sequences->sets[i]->neighbors[j]->indexes,
		    int,
		    nb_neighbors + 1);
	      m->sets_of_sequences->sets[i]->neighbors[j]->indexes[nb_neighbors] =
		k;
	      nb_neighbors++;
	    }
	  }
	}

	m->sets_of_sequences->sets[i]->neighbors[j]->nb_neighbors = 
	  nb_neighbors;

      }
   

      /*Initialize the maxi clique for the current set*/
      NEW(m->sets_of_sequences->sets[i]->clique_set,
	  clique_s,
	  1);
    
      m->sets_of_sequences->sets[i]->clique_set->clique_index = NULL;
      m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique = 0;
      is_finished = 0;

      /*Search recursively for the maxi clique for the current set*/
      for(j = 0; !is_finished && 
	    j < m->sets_of_sequences->sets[i]->nb_q_intervals; j++){

	/*Stop condition : find a maxi clique that has enough
	  sequences if a number of sequences by set has been provided
	  otherwise continue until all sequences has been processed*/
	if(((OPTS_nb_seqs_set != -1) &&
	    (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique < 
	     OPTS_nb_seqs_set - 1)) ||
	   ((OPTS_nb_seqs_set == -1) &&
	    (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique !=
	     m->sets_of_sequences->sets[i]->nb_q_intervals))){

	  NEW(temp_clique,
	      clique_s,
	      1);
	  temp_clique->clique_index = NULL;
	  temp_clique->nb_seqs_clique = 0;
	  recursive_search_maxi_clique(m->sets_of_sequences->sets[i], 
				       temp_clique,
				       j);
	  DESTROY(temp_clique->clique_index);
	  DESTROY(temp_clique);

	}
	else{
	  is_finished = 1;
	}
      
      }

      if(OPTS_verbose){
      
	print_verbose_end_maxi_clique_set(m->sets_of_sequences->
					  sets[i]->clique_set->nb_seqs_clique + 1);
	
      }

    }

  }
}
