////////////////////////////////////////////////////////////////////////////
///
/// \file matt_ygraph.h
///
/// \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
///
/// \brief Header file that contains all functions used to retrieve 
/// conserved regions and select some
///
/// \version 1.02
///
/// \date September 2008
///
///
/// This file will define all functions that will retrieve conserved regions
/// according to alignments and all functions used to select sequences
/// according to users parameters
///
///
/////////////////////////////////////////////////////////////////////////////


#ifndef __MATT_YGRAPH_H__
#define __MATT_YGRAPH_H__


/*Intern inclusions*/
#include "types.h"



///////////////////////////////////////////////////////////////////////////
///
/// \fn float score_function(float score_pos_i_minus_1,
///                          int nb_al_pos_i,
///                          float mean_al,
///                          float epsilon,
///                          int lambda)
///
/// \brief Defines the score function for the position i
///
/// \param[in] score_pos_i_minus_1 : the score at position i - 1
/// \param[in] nb_al_pos_i : number of alignments at position i
/// \param[in] mean_al : mean number of alignments by position
/// \param[in] epsilon : float to add at the nb_al_i parameter to avoid
/// log(0)
/// \param[in] lambda : parameter threshold
///
/// \return The score at the position i
///
/// \note Si = max(0,Si-1 + log(ni + espilon / lanmba A)
///          Si : score at position i
///          Si-1 : score at position i-1
///          ni : number of alignments at position i$
///          epsilon : float to avoid log(0) in cas ni = 0
///          A : mean number of alignments
///          lambda : parameter threshold
///
/// This function will calculate the score by position
///
///////////////////////////////////////////////////////////////////////////


extern float score_function(float score_pos_i_minus_1,		
			    int  nb_al_pos_i,			
			    float  mean_al,				
			    float  epsilon,				
			    int  lambda);

////////////////////////////////////////////////////////////////////////////
///
/// \fn void destroy_main_structure(p_matt_ygraph m)
/// 
/// \brief Function used to destroy all structures allocated
///
/// \param[in,out] m : the main structure to destroy
///
/// That function will destroy all allocated memory according to mode used
///
/////////////////////////////////////////////////////////////////////////////


extern void destroy_main_structure(p_matt_ygraph m);


////////////////////////////////////////////////////////////////////////////
///
/// \fn int is_present_int_array(int num, int *array, int length)
/// 
/// \brief Function used to check if a number is present in an array of
/// integers
///
/// \param[in] num : the number to search for
/// \param[in] array : the array of integers
/// \param[in] length : length of the array
///
/// \return 1 if the number was found, 0 otherwise
///
/// That function will check if a number is present in an array of integres
/// and return 1 if  it was found and 0 in the other case
///
/////////////////////////////////////////////////////////////////////////////

extern int is_present_int_array(int num, int *array, int length);


////////////////////////////////////////////////////////////////////////////
///
/// \fn int is_present_char_array(char *string, char_array *array)
/// 
/// \brief Function used to check if a string is present in a char_array
///
/// \param[in] string : the string to search for
/// \param[in] array : the char_array
///
/// \return 1 if the string was found, 0 otherwise
///
/// That function will check if a string is present in an array of strings
/// (char_array) and return 1 if  it was found and 0 in the other case
///
/////////////////////////////////////////////////////////////////////////////

extern int is_present_char_array(char *string, char_array *array);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void copy_interval_query(p_interval_query interval_1,
///                              p_interval_query interval_2)
/// 
/// \brief Function used to copy two interval queries
///
/// \param[in,out] interval_1 : the interval to copy in
/// \param[in] interval_2 : the interval to copy
///
/// That function will copy all the attributes of the interavl_2 in the 
/// first one
///
/////////////////////////////////////////////////////////////////////////////

extern void copy_interval_query(p_interval_query interval_1,
				p_interval_query interval_2);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void calculate_nb_al_function(p_matt_ygraph m)
/// 
/// \brief Function used to calculate the number of alignments by position
///
/// \param[in,out] m : the main structure containing all needed information
///
/// That function will count the number of alignments by position and fill
/// the corresponding array
///
/////////////////////////////////////////////////////////////////////////////

extern void calculate_nb_al_function(p_matt_ygraph m);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void calculate_score_function(p_matt_ygraph m)
/// 
/// \brief Function used to calculate the score function
///
/// \param[in,out] m : the main structure to destroy
///
/// That function will calculate the score function used to retrieve
/// conserved region
///
/////////////////////////////////////////////////////////////////////////////

extern void calculate_score_function(p_matt_ygraph m);



////////////////////////////////////////////////////////////////////////////
///
/// \fn void get_interesting_regions(p_matt_ygraph m)
///
/// \brief Function used to determine interesting regions on the reference
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated with defined regions
///
/// That function will use the scores calculated by the score function to
/// determine positions of conserved regions on the reference sequence
///
/////////////////////////////////////////////////////////////////////////////

extern void get_interesting_regions(p_matt_ygraph m);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void determine_pos_query(p_matt_ygraph m, 
///                              int index_interval, 
///                              int index_al)
///
/// \brief Function used to determine the positions on the query sequence
/// for the conserved region on the reference
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
/// \param[in] index_interval : index of the reference interval in the array
/// of conserved region
/// \param[in] index_al : index of the alignment concerned 
///
/// That function will determine the positions on the query sequence for the 
/// the conserved region on the reference. It points at the correct interval
/// by the index_interval parameter and the correct alignment by the index_al
/// parameter.
///
/////////////////////////////////////////////////////////////////////////////


extern void determine_pos_query(p_matt_ygraph m, 
				int index_interval, 
				int index_aln);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void complete_set_sequences(p_matt_ygraph m)
///
/// \brief Function used to determine the sequences of the reference 
/// sequence and all query sequences of all sets
///
/// \param[in,out] m : main structure containing all sets
///
/// That function will determine the sequences of the reference 
/// sequence and all query sequences according to positions and strand on all
/// sets of sequences
///
/////////////////////////////////////////////////////////////////////////////


extern void complete_set_sequences(p_matt_ygraph m);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void get_sets_of_sequences(p_matt_ygraph m)
///
/// \brief Function used to determine the sets of sequences of conserved 
/// regions
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated with the sets
///
/// That function will determine the positions on query sequences according
/// to the positions of conserved regions on the reference and the info
/// about alignments
///
/////////////////////////////////////////////////////////////////////////////


extern void get_sets_of_sequences(p_matt_ygraph m);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void eliminate_redundancy_set(p_matt_ygraph m, int index_set)
///
/// \brief Function used to eliminate redundant sequences inside the 
/// current set
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
/// \param[in] index_set : the index of the set to process
///
/// That function will eliminate redundant sequences inside the set (same
/// species and overlapped positions) 
///
/////////////////////////////////////////////////////////////////////////////


extern void eliminate_redundancy_set(p_matt_ygraph m, int index_set);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void compare_all_against_ref(p_matt_ygraph m)
///
/// \brief Function used to compare all sequences against the reference
/// for all sets of sequences
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
///
/// That function will compare all sequences against the reference
/// for all sets of sequences and eliminate all queries that don't repsect
/// the identity thresholds (min and max)
///
/////////////////////////////////////////////////////////////////////////////


extern void compare_all_against_ref(p_matt_ygraph m);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void sort_identity_ref(p_matt_ygraph m)
///
/// \brief Function used to sort all queries in each set by reference 
/// identity
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
///
/// Function used to sort all queries in each set by reference 
/// identity
///
/////////////////////////////////////////////////////////////////////////////


extern void sort_identity_ref(p_matt_ygraph m);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void check_species_repetitions(p_matt_ygraph m)
///
/// \brief Function used to check species repetitions
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
///
/// That function will check if the species repetitions threshold is
/// respected
///
/////////////////////////////////////////////////////////////////////////////

extern void check_species_repetitions(p_matt_ygraph m);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void search_naive_clique(p_matt_ygraph m)
///
/// \brief Function used to search the naive clique for all sets
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
///
/// That function will search the naive clique for all sets. It will include
/// by reference identity order each sequences and abort as soon as one
/// cannot be included
///
/////////////////////////////////////////////////////////////////////////////


extern void search_naive_clique(p_matt_ygraph m);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void recursive_search_maxi_clique(p_set_of_seqs set,
///                                       p_clique temp_clique,
///                                       int index)
///
/// \brief Function used to search recursively the maxi clique in the
/// current set
///
/// \param[in,out] set : the current set
/// \param[in,out] temp_clique : the temporary maxi clique
/// \param[in] index : the index in the query array of the sequence to add 
///
/// That function will search the maxi clique recursively by trying to 
/// add the query sequence in the temporary maxi clique
///
/////////////////////////////////////////////////////////////////////////////

extern void recursive_search_maxi_clique(p_set_of_seqs set,
					 p_clique temp_clique,
					 int index);


////////////////////////////////////////////////////////////////////////////
///
/// \fn void maxi_clique_search(p_matt_ygraph m)
///
/// \brief Function used to search the maxi clique for all sets
///
/// \param[in,out] m : the main structure containing information and that
/// will be updated
///
/// That function will search the maxi clique search for all sets. If a
/// number of sequences has been provided by the user the maxi clique will
/// be at least that size
///
/////////////////////////////////////////////////////////////////////////////

extern void maxi_clique_search(p_matt_ygraph m);




#endif
