/////////////////////////////////////////////////////////////////////////////
///
/// \file options.c
///
/// \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
///
/// \brief File that defines the software options
///
/// \version 1.02
///
/// \date September 2008
///
///
/// This file will define all options that can be used in the software
///
///
/////////////////////////////////////////////////////////////////////////////


/*Basic inclusions*/
#include <getopt.h>


/*Intern inclusions*/
#include "options.h"
#include "macros.h"
#include "io.h"

/******************************************************************************
 **
 **                         MAIN OPTIONS
 **     
 *****************************************************************************/


/******************************************************************************
 **   Reference file
 **     
 *****************************************************************************/

char *OPTS_ref_file = NULL; 


/******************************************************************************
 **   Other species file(s)
 **     
 *****************************************************************************/

char *OPTS_file;


/******************************************************************************
 **   Array of other file sequences in fasta format
 **     
 *****************************************************************************/

char_array *OPTS_file_array = NULL;


/******************************************************************************
 **   Output directory (Default : current directory)
 **     
 *****************************************************************************/

char *OPTS_output_dir = "./";


/******************************************************************************
 **   Output format (Default : GFF3 ( = 1)
 **     
 *****************************************************************************/

int OPTS_output_format = 1; 


/******************************************************************************
 **   Verbose option (Default : no)
 **     
 *****************************************************************************/

int OPTS_verbose = 0;

/******************************************************************************
 **
 **                         FINDING CONSERVED REGIONS OPTIONS
 **     
 *****************************************************************************/



/******************************************************************************
 **   Lambda parameter : the paramter lambda used in the calculation of
 **   scores by position (Default : 1)
 **     
 *****************************************************************************/

int OPTS_lambda = 1; 



/******************************************************************************
 **   Array of alignments files
 **     
 *****************************************************************************/

char_array *OPTS_alignments = NULL;


/******************************************************************************
 **   Minimal length of a region (Default : 30)
 **     
 *****************************************************************************/

int OPTS_min_length_region = 30;

/******************************************************************************
 **   Maximal length of a region (Default : 500)
 **     
 *****************************************************************************/

int OPTS_max_length_region = 500;


/******************************************************************************
 **   Filter on species (Default : No)
 **     
 *****************************************************************************/

int OPTS_filter_species = 0;


/******************************************************************************
 **
 **                         SELECTING SEQUENCES
 **     
 *****************************************************************************/


/******************************************************************************
 **   Minimum identity threshold (Default 50%)
 **     
 *****************************************************************************/

int OPTS_min_id = 50; 


/******************************************************************************
 **   Maximum identity threshold (Default 100%)
 **     
 *****************************************************************************/

int OPTS_max_id = 100; 


/******************************************************************************
 **   Number of sequences by set (reference + queries) (Default : all == -1)
 **     
 *****************************************************************************/

int OPTS_nb_seqs_set = -1; 


/******************************************************************************
 **   Selecting mode (Default 0 -> etoile)
 **     
 *****************************************************************************/

int OPTS_selecting_mode = 0; 


/******************************************************************************
 **   Maximum number of repetitions by species (Default -1 -> not used)
 **     
 *****************************************************************************/

int OPTS_max_species_repet = -1;  


/******************************************************************************
 **   Carnac selected parameter (Default 0 -> not used)
 **     
 *****************************************************************************/

int OPTS_carnac = 0;


/******************************************************************************
 **   Which program use : arnica (default) or carnac
 **     
 *****************************************************************************/

char *PROG_CARNAC = NULL;



/******************************************************************************
 **
 **                         FUNCTIONS
 **     
 *****************************************************************************/


/////////////////////////////////////////////////////////////////////////////
///
/// \fn void usage()
///
/// \brief Function that defines the software options
///
/// This function will be printed on the screen to help users deal with 
/// options. It will be printed also when user's options are not correct
///
///
/////////////////////////////////////////////////////////////////////////////


void usage()
{

  fprintf(DISPLAY_ERR,"\nUSAGE:\n");
  fprintf(DISPLAY_ERR,"  " PACKAGE_NAME " -r <reference file> -f <other sequences file> <alignments \"blast tabulated\" files> [options]\n\n");
  fprintf(DISPLAY_ERR,"     -h        Display this help\n");
  fprintf(DISPLAY_ERR,"     -v        Verbose\n");
  fprintf(DISPLAY_ERR,"     -r <str>  Reference file in fasta format\n");
  fprintf(DISPLAY_ERR,"     -f <str>  File(s) in fasta format containing other sequences separated by \",\"\n");
  fprintf(DISPLAY_ERR,"                Or\n");
  fprintf(DISPLAY_ERR,"               One multifasta file containg all other sequences\n");
  fprintf(DISPLAY_ERR,"               (See examples below for usage)\n");
  fprintf(DISPLAY_ERR,"     -d <str>  Directory output (Default : write in current directory)\n");
  fprintf(DISPLAY_ERR,"     -o <int>  Output format :\n");
  fprintf(DISPLAY_ERR,"                1 : GFF3 (Default)\n");
  fprintf(DISPLAY_ERR,"                2 : Multifasta\n\n");
  fprintf(DISPLAY_ERR,"  Examples :\n");
  fprintf(DISPLAY_ERR,"     Example with one reference and two other species in separated fasta files :\n");
  fprintf(DISPLAY_ERR,"  matt_ygraph -r reference.fna -f species_1.fna,species_2.fna ali_ref_species_1.blast ali_ref_species_2.blast\n\n"); 
  fprintf(DISPLAY_ERR,"     Example with one reference and two other species in one multifasta file :\n");
  fprintf(DISPLAY_ERR,"  matt_ygraph -r reference.fna -f species_1_2.fna ali_ref_species_1.blast ali_ref_species_2.blast\n\n");
  fprintf(DISPLAY_ERR,"  1) Find conserved regions from alignments options :\n");
  fprintf(DISPLAY_ERR,"     -s <int>  Score lambda parameter (Default : 1)\n");
  fprintf(DISPLAY_ERR,"     -L <int>  Maximal length of a conserved region (Default : 500)\n");
  fprintf(DISPLAY_ERR,"     -l <int>  Minimal length of a conserved region (Default : 30)\n");
  fprintf(DISPLAY_ERR,"     -F        Filter redundant alignment intra species (Default : No)\n");
  fprintf(DISPLAY_ERR,"     -R <int>  Maximum number of repetitions by species (Default : not used)\n\n");
  fprintf(DISPLAY_ERR,"  2) Select sequences according to parameters options :\n");
  fprintf(DISPLAY_ERR,"     -i <int>  Minimum identity threshold in percent (Default : 50)\n");
  fprintf(DISPLAY_ERR,"     -I <int>  Maximum identity threshold in percent (Default : 100)\n");
  fprintf(DISPLAY_ERR,"     -c        Common secondary structure shared (using Arnica or Carnac)\n");
  fprintf(DISPLAY_ERR,"     -M <int>  Selecting mode\n");
  fprintf(DISPLAY_ERR,"                0 : etoile (Default)\n");
  fprintf(DISPLAY_ERR,"                1 : naive clique\n");
  fprintf(DISPLAY_ERR,"                2 : maxi clique\n");
  fprintf(DISPLAY_ERR,"     -n <int>  Number of sequences wanted by set (ref + queries) (Default : all)\n\n");
  
}


/////////////////////////////////////////////////////////////////////////////
///
/// \fn int parse_params(int argc, char **argv)
///
/// \brief Function used to parse user parameters
///
/// \param[in] argc : number of parameters
/// \param[in] argv : the parameters
///
/// \return an integer representing the status : \n
/// 1 -> exit success \n
/// 0 -> an error occured
///
/// This function will parse all user parameters and will check for their 
/// validity
///
///
/////////////////////////////////////////////////////////////////////////////


int parse_params(int argc, char **argv)
{
  /*Status*/
  int status = 1;
  /*The temporary character read*/
  char char_read;
  /*Number of parameters read*/
  int nb_param = 0;
  /*Index*/
  int i;

  /*Don't print automatic error messages*/
  opterr = 0;

  /*Read the parameters*/
  while(((char_read = getopt(argc, argv, "hvcFr:f:s:d:L:l:i:I:n:M:R:o:")) != EOF) 
	&&
	(status != 2)){
 
    switch(char_read) {
      /*Help case*/
    case 'h':     
      status = 2;
      nb_param += 1;
      break;

      /*Verbose case*/
    case 'v':     
      OPTS_verbose = 1;
      nb_param += 1;
      break;

      /*Output format*/
    case 'o':
      OPTS_output_format = atoi(optarg);
      if((OPTS_output_format < 1) ||
	 (OPTS_output_format > 2)){
	ERROR_ "The output format must be 1 (GFF3) or 2(Multifasta)\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;
      
      /*Reference case*/
    case 'r' :
      NEW(OPTS_ref_file, char, strlen(optarg) + 1);
      strcpy(OPTS_ref_file, optarg);
      nb_param += 2;
      break;

      /*File(s) containing sequences case*/
    case 'f' : 
      NEW(OPTS_file, char, strlen(optarg) + 1);
      strcpy(OPTS_file, optarg);
      nb_param += 2;
      break;

      /*Score lambda case*/
    case 's' :
      OPTS_lambda = atoi(optarg);
      if(OPTS_lambda <= 0){
	ERROR_ "The lambda score must be higher than 0\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Directory output case*/
    case 'd' :
      /*Check if there's a "/" at the end of the optarg*/
      if(optarg[strlen(optarg) - 1] == '/'){
	NEW(OPTS_output_dir, char, strlen(optarg) + 1);
	strcpy(OPTS_output_dir, optarg);
      }
      else{
	NEW(OPTS_output_dir, char, strlen(optarg) + 2);
	sprintf(OPTS_output_dir,
		"%s%s",
		optarg,
		"/");
      }
      
      nb_param += 2;
      break;

      /*Maximal length of a conserved region case*/
    case 'L' :
      OPTS_max_length_region = atoi(optarg);
      if(OPTS_max_length_region <= 0){
	ERROR_ "The maximal length of a conserved region must be positive\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Minimal length of a conserved region case*/
    case 'l' :
      OPTS_min_length_region = atoi(optarg);
      if(OPTS_min_length_region <= 0){
	ERROR_ "The minimal length of a conserved region must be positive\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Carnac case*/
    case 'c' :
      OPTS_carnac = 1;
      nb_param += 1;
      break;

      /*Filter redundant alignment intra species case*/
    case 'F' :
      OPTS_filter_species = 1;
      nb_param += 1;
      break;

      /*Minimum identity threshold case*/
    case 'i' :
      OPTS_min_id = atoi(optarg);
      if((OPTS_min_id <= 0) || (OPTS_min_id >= 100)){
	ERROR_ "The minimal identity threshold must be bewteen 0 and 99\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Maximum identity threshold case*/
    case 'I' :
      OPTS_max_id = atoi(optarg);
      if((OPTS_max_id <= 1) || (OPTS_max_id > 100)){
	ERROR_ "The maximal identity threshold must be between 1 and 100\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Selecting mode case*/
    case 'M' :
      OPTS_selecting_mode = atoi(optarg);
      if((OPTS_selecting_mode < 0) || (OPTS_selecting_mode > 2)){
	ERROR_ "The selecting mode must be 0 (etoile), 1(naive clique) or 2 (maxi clique)\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Number of sequences by set case*/
    case 'n' :
      OPTS_nb_seqs_set = atoi(optarg);
      if(OPTS_nb_seqs_set <= 1){
	ERROR_ "The number of sequences wanted by set must be above 2 (ref + 1 query)\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

   /*Maximum number of repetitions by organism*/
    case 'R' :
      OPTS_max_species_repet = atoi(optarg);
      if(OPTS_max_species_repet < 1){
	ERROR_ "The maximum number of repeptitions by species must be at least 1\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;   

      /*Other cases*/
    case '?' :
      if ((optopt == 'o') || (optopt == 'r') || (optopt == 'f') ||
	  (optopt == 's') || (optopt == 'd') || (optopt == 'l') ||
	  (optopt == 'L') || (optopt == 'i') || (optopt == 'I') ||
	  (optopt == 'n') || (optopt == 'R')){
	ERROR_ "Option -%c requires an argument\n\n",optopt _ERROR;
      }
      else{
	ERROR_ "Unrecognized option -%c\n\n",optopt _ERROR;
      }
      status = 2;
      break;
    }

   
  }

  if(status != 2){
   
    /*Initialize the array*/
    NEW(OPTS_alignments, char_array, 1);
    OPTS_alignments->chars = NULL;
    OPTS_alignments->nb_chars = 0;
    
    /*Get all alignments files*/
    for(i = 0; i < argc - nb_param -1; i++){
      RENEW(OPTS_alignments->chars, 
	    char*, 
	    OPTS_alignments->nb_chars + 1);
      NEW(OPTS_alignments->chars[OPTS_alignments->nb_chars],
	  char,
	  strlen(argv[optind + i]) + 1);
      strcpy(OPTS_alignments->chars[OPTS_alignments->nb_chars],
	     argv[optind + i]);
      OPTS_alignments->nb_chars++;

    }
 
  }


  /*Return the status*/
  return status;

}


/////////////////////////////////////////////////////////////////////////////
///
/// \fn int check_parameters_validity
///
/// \brief Function used to check parameters entered by the user
///
/// \return an integer representing the validity : \n
/// 1 -> exit success \n
/// 0 -> an error occured
///
/// This function will check parameters validity and returns the result of
/// the checking
///
///
/////////////////////////////////////////////////////////////////////////////

int check_parameters_validity()
{

  /*Index*/
  int i;
  /*File descriptor*/
  FILE *fd;
  /*Command line to create a directory*/
  char *command_line_dir = NULL;
  /*Error code for the commande line*/
  int err_code;
  /*Boolean that says if carnac is available*/
  int is_carnac_present = 1;
  /*Boolean that says if arnica is available*/
  int is_arnica_present = 1;
  /*String used to parse other species file*/
  char *s_parse = NULL;

  long last_pos = 0;

  /*Check if there's at least one reference file and one alignment file*/
  if(OPTS_ref_file == NULL){
    ERROR_ "You must give a reference file\n\n" _ERROR;
    return 0;
  }
  
  if(OPTS_file == NULL){
    ERROR_ "You must give at least one other file\n\n" _ERROR; 
    return 0;
  }

  /*Check if reference file provided exist*/
  fd = fopen(OPTS_ref_file, "r");
  if (fd == NULL){
    ERROR_ "The reference file %s doesn't exist\n\n",OPTS_ref_file _ERROR;
    return 0;
  }
  fclose(fd);

  /*Get the other sequence file(s)*/
  s_parse = strchr(OPTS_file, ',');
  NEW(OPTS_file_array, char_array, 1);
  OPTS_file_array->chars = NULL;
  OPTS_file_array->nb_chars = 0;

  while(s_parse != NULL){

    RENEW(OPTS_file_array->chars,
	  char*,
	  OPTS_file_array->nb_chars + 1);
    NEW(OPTS_file_array->chars[OPTS_file_array->nb_chars],
	char,
	last_pos + (s_parse - OPTS_file) + 1);
    
    for(i = last_pos; i < s_parse - OPTS_file; i++){
      OPTS_file_array->chars[OPTS_file_array->nb_chars][i - last_pos] = 
	OPTS_file[i];
    }

    OPTS_file_array->chars[OPTS_file_array->nb_chars][i - last_pos] = '\0';
    last_pos = i + 1;
    
    OPTS_file_array->nb_chars++;
    s_parse = strchr( s_parse + 1, ',');
  }

  s_parse = strchr(OPTS_file, '\0');
  if(s_parse != NULL){

    RENEW(OPTS_file_array->chars,
	  char*,
	  OPTS_file_array->nb_chars + 1);
    NEW(OPTS_file_array->chars[OPTS_file_array->nb_chars],
	char,
	last_pos + (s_parse - OPTS_file) + 1);
    
    for(i = last_pos; i < s_parse - OPTS_file; i++){
      OPTS_file_array->chars[OPTS_file_array->nb_chars][i - last_pos] = 
	OPTS_file[i];
    }

    OPTS_file_array->chars[OPTS_file_array->nb_chars][i - last_pos] = '\0';
    last_pos = i + 1;  
    OPTS_file_array->nb_chars++;
  }


  /*Check if other species file(s) exists*/
  for(i = 0; i < OPTS_file_array->nb_chars; i++){
    fd = fopen(OPTS_file_array->chars[i], "r");
    if(fd == NULL){
      ERROR_ "The sequence file %s doesn't exist\n\n",
	OPTS_file_array->chars[i] _ERROR;
      return 0;

    }
    fclose(fd);
  }


  /*Check if alignments file(s) exist*/
  for(i = 0; i < OPTS_alignments->nb_chars; i++){
    fd = fopen(OPTS_alignments->chars[i], "r");
    if(fd == NULL){
      ERROR_ "The alignment file %s doesn't exist\n\n",
	OPTS_alignments->chars[i] _ERROR;
      return 0;

    }
    fclose(fd);
  }
  

  /*Check if output directory exists : if not create it*/
   NEW(command_line_dir, 
      char,
      2 * strlen(OPTS_output_dir) + 
      strlen("if [ ! -d ") +
      strlen(" ] ; then mkdir ") +
      strlen("; fi;") + 1);
   sprintf(command_line_dir,
	   "%s%s%s%s%s",
	   "if [ ! -d ",
	   OPTS_output_dir,
	   " ] ; then mkdir ",
	   OPTS_output_dir,
	   "; fi;");
   
   err_code = system(command_line_dir);
   DESTROY(command_line_dir);
   
   /*Check if min_id is below max_id*/
   if(OPTS_min_id >= OPTS_max_id){
     ERROR_ "The mimumum identity threshold must be lower than the maximum\n\n"
       _ERROR;
     return 0;
   }

   /*Check if max length of a conserved region is above the min length*/
   if(OPTS_min_length_region > OPTS_max_length_region){
     ERROR_ "The mimumum length of a conserved region  must be lower than the maximum\n\n"
       _ERROR;
     return 0;

   }

   /*Check if Arnica or carnac are available*/
   if(OPTS_carnac){
     
     /*Look which program is available*/
     if(strcmp(CARNAC_EXE,"") == 0){
       is_carnac_present = 0;
     }

     if(strcmp(ARNICA_EXE,"") == 0){
       is_arnica_present = 0;
     }

     if((!is_carnac_present) && (!is_arnica_present)){

       ERROR_ "Neither Arnica nor Carnac are available : the option -c cannot be used\n\n"
	 _ERROR;
       return 0;

     }
     
     /*Determine wich program to use : arnica (default) or carnac*/
     if(is_arnica_present){
       PROG_CARNAC = "arnica";
     }
     else{
       PROG_CARNAC = "carnac";
     }
 
   }

  /*Return success*/
  return 1;


}


/////////////////////////////////////////////////////////////////////////////
///
/// \fn void destroy_options
///
/// \brief Function used to destroyed allocated options
///
/// This function will destroy the allocated options : the reference name,
/// the array of alignments files and the location of the file in fasta or 
/// in GFF3 format
///
///
/////////////////////////////////////////////////////////////////////////////

void destroy_options()
{
  
  /*Index*/
  int i;

  /*Destroy the reference file*/
  if(OPTS_ref_file){
    DESTROY(OPTS_ref_file);
  }

  /*Destroy the file*/
  if(OPTS_file){
    DESTROY(OPTS_file);
  }

  /*Destroy the array of sequence file*/
  if(OPTS_file_array){
    for(i = 0; i < OPTS_file_array->nb_chars; i++){
      DESTROY(OPTS_file_array->chars[i]);
    }
    DESTROY(OPTS_file_array->chars);
    DESTROY(OPTS_file_array);

  }

  /*Destroy the array of alignments file*/
  if(OPTS_alignments){
    for(i = 0; i < OPTS_alignments->nb_chars; i++){
      DESTROY(OPTS_alignments->chars[i]);
    }
    DESTROY(OPTS_alignments->chars);
    DESTROY(OPTS_alignments);

  }

  /*Destroy output dir*/
  if(OPTS_output_dir){
    if(strcmp(OPTS_output_dir, "./") != 0){
      DESTROY(OPTS_output_dir);
    }
  }


}
