/////////////////////////////////////////////////////////////////////////////
///
/// \file options.h
///
/// \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
///
/// \brief Header file that defines the software options
///
/// \version 1.02
///
/// \date September 2008
///
///
/// This file will define all options that can be used in the software
///
///
/////////////////////////////////////////////////////////////////////////////


#ifndef __OPTIONS_H__
#define __OPTIONS_H__

/*Intern inclusions*/
#include "types.h"




/******************************************************************************
 **
 **                         MAIN OPTIONS
 **     
 *****************************************************************************/


/******************************************************************************
 **   Reference file
 **     
 *****************************************************************************/

extern char *OPTS_ref_file; /*!< Reference file*/


/******************************************************************************
 **   Other species file(s)
 **     
 *****************************************************************************/

extern char *OPTS_file; /*!< Other species file(s)*/


/******************************************************************************
 **   Array of other file sequences in fasta format
 **     
 *****************************************************************************/

extern char_array *OPTS_file_array; /*!< Array of file(s) containing sequences*/


/******************************************************************************
 **   Output directory
 **     
 *****************************************************************************/

extern char *OPTS_output_dir; /*!< Output directory*/


/******************************************************************************
 **   Output format
 **     
 *****************************************************************************/

extern int OPTS_output_format; /*!< Output format*/



/******************************************************************************
 **   Verbose option
 **     
 *****************************************************************************/

extern int OPTS_verbose; /*!< Verbose option*/


/******************************************************************************
 **
 **                         FINDING CONSERVED REGIONS OPTIONS
 **     
 *****************************************************************************/


/******************************************************************************
 **   Lambda parameter : the paramter lambda used in the calculation of
 **   scores by position 
 **     
 *****************************************************************************/

extern int OPTS_lambda; /*!< Score lambda parameter*/


/******************************************************************************
 **   Array of alignments files
 **     
 *****************************************************************************/

extern char_array *OPTS_alignments; /*!< Array of alignments file*/


/******************************************************************************
 **   Minimal length of a region
 **     
 *****************************************************************************/

extern int OPTS_min_length_region; /*!< Minimal length of a region*/


/******************************************************************************
 **   Maximal length of a region
 **     
 *****************************************************************************/

extern int OPTS_max_length_region; /*!< Maximal length of a region*/


/******************************************************************************
 **   Filter on species : an alignment will be take into account only one
 **   time for a same species
 **     
 *****************************************************************************/

extern int OPTS_filter_species; /*!< Filter on species */


/******************************************************************************
 **
 **                         SELECTING SEQUENCES
 **     
 *****************************************************************************/


/******************************************************************************
 **   Minimum identity threshold 
 **     
 *****************************************************************************/

extern int OPTS_min_id; /*!< Minimum identity threshold*/


/******************************************************************************
 **   Maximum identity threshold 
 **     
 *****************************************************************************/

extern int OPTS_max_id; /*!< Minimum identity threshold*/


/******************************************************************************
 **   Number of sequences by set (reference + queries)
 **     
 *****************************************************************************/

extern int OPTS_nb_seqs_set; /*!< Number of sequences wanted by set*/


/******************************************************************************
 **   Selecting mode (Default 0 -> etoile)
 **     
 *****************************************************************************/

extern int OPTS_selecting_mode;  /*!< Selecting mode*/ 


/******************************************************************************
 **   Maximum number of repetitions by species (Default -1 -> not used)
 **     
 *****************************************************************************/

extern int OPTS_max_species_repet;  /*!< Maximum number of repetitions by species*/ 


/******************************************************************************
 **   Carnac selected parameter (Default 0 -> not used)
 **     
 *****************************************************************************/

extern int OPTS_carnac; /*!< Carnac selected parameter*/


/******************************************************************************
 **   Which program use : arnica (default) or carnac
 **     
 *****************************************************************************/

char *PROG_CARNAC; /*!< Which program to use : arnica or carnac*/


/******************************************************************************
 **
 **                         FUNCTIONS
 **     
 *****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
///
/// \fn void usage()
///
/// \brief Function that defines the software options
///
/// This function will be printed on the screen to help users deal with 
/// options. It will be printed also when user's options are not correct
///
///
/////////////////////////////////////////////////////////////////////////////


extern void usage();


/////////////////////////////////////////////////////////////////////////////
///
/// \fn int parse_params(int argc, char **argv)
///
/// \brief Function used to parse user parameters
///
/// \param[in] argc : number of parameters
/// \param[in] argv : the parameters
///
/// \return an integer representing the status : \n
/// 1 -> exit success \n
/// 0 -> an error occured
///
/// This function will parse all user parameters and will check for their 
/// validity
///
///
/////////////////////////////////////////////////////////////////////////////


extern int parse_params(int argc, char **argv);



/////////////////////////////////////////////////////////////////////////////
///
/// \fn int check_parameters_validity
///
/// \brief Function used to check parameters entered by the user
///
/// \return an integer representing the validity : \n
/// 1 -> exit success \n
/// 0 -> an error occured
///
/// This function will check parameters validity and returns the result of
/// the checking
///
///
/////////////////////////////////////////////////////////////////////////////

extern int check_parameters_validity();



/////////////////////////////////////////////////////////////////////////////
///
/// \fn void destroy_options
///
/// \brief Function used to destroyed allocated options
///
/// This function will destroy the allocated options : the reference name,
/// the array of alignments files and the location of the file in fasta or 
/// in GFF3 format
///
///
/////////////////////////////////////////////////////////////////////////////

extern void destroy_options();

#endif
