////////////////////////////////////////////////////////////////////////////
///
/// \file sequence.c
///
/// \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
///
/// \brief File that defines all functions related to sequences
///
/// \version 1.02
///
/// \date September 2008
///
///
/// This file will define all fucntions related to sequences  
///
///
/////////////////////////////////////////////////////////////////////////////

/*Basic inclusions*/
#include <ctype.h>


/*Intern inclusions*/
#include "sequence.h"
#include "options.h"
#include "sequence_interface.h"
#include "io.h"
#include "macros.h"


/////////////////////////////////////////////////////////////////////////////
///
/// \fn void get_sequences(sequence_array *array_of_seqs)
///
/// \brief Function used to retrieve sequences contained in the fasta file
/// and reference file
///
/// \param[in,out] array_of_seqs : the array of sequences that will be
/// updated with sequences contained in the fasta file
///
///
/// This function will use the sequence parser to retrieve the sequences in
/// the fasta file
///
///
/////////////////////////////////////////////////////////////////////////////

void get_sequences(sequence_array *array_of_seqs)
{

  /*Indexes*/
  int i,j;
  /*File descriptor*/
  FILE *fd = NULL;
  /*Temporary sequence array*/
  p_sequence *other_sequences = NULL;
  /*Number of sequences*/
  int nb_seqs = 0;

  

  /*Try to read the reference file*/
  if((fd = fopen(OPTS_ref_file, "r")) == NULL) {
    ERROR_ "Unable to open '%s' for reading...\n",OPTS_ref_file _ERROR;
    array_of_seqs->seqs = NULL;
    array_of_seqs->nb_seqs = 0;
  }
  
  /*Parse the file to extract the sequences*/
  sequence_file_name = OPTS_ref_file;   
  sequencerestart(fd); 
  sequenceparse(); 
  
  fclose(fd);
  free_flex_mem();

  /*If there were some errors print error message, return NULL
   and free all allocated memory from the parser*/
  if(sequence_nb_errors != 0) {
    ERROR_ "File '%s' has some errors : process stopped\n\n",OPTS_file _ERROR; 
    array_of_seqs->seqs = NULL;
    array_of_seqs->nb_seqs = 0;
    i = 0;
    while(sequence_result[i] != NULL){
      if(sequence_result[i]->name != NULL){
	DESTROY(sequence_result[i]->name);
      }
      if(sequence_result[i]->bases != NULL){
	DESTROY(sequence_result[i]->bases);
      }
      DESTROY(sequence_result[i]);
      i++;
    }
    DESTROY(sequence_result);
  }
  else{

    /*Retrievment of the sequences array*/
    array_of_seqs->seqs = sequence_result;
  
    /*Counting the number of sequences*/
    while(array_of_seqs->seqs[nb_seqs] != NULL) nb_seqs++;

    /*If there was not enough sequences return NULL*/
    if(nb_seqs < 1) {
      ERROR_ "No reference sequences....\n" _ERROR;
      array_of_seqs->seqs = NULL;
      array_of_seqs->nb_seqs = 0;
    }
    else{
  
      array_of_seqs->nb_seqs = nb_seqs;
      nb_seqs = 0;

      /*Try to read the files containing other sequences*/
      for(j = 0; j < OPTS_file_array->nb_chars; j++){

      /*Parse the file to extract the sequences*/
	if((fd = fopen(OPTS_file_array->chars[j], "r")) == NULL) {
	  ERROR_ "Unable to open '%s' for reading...\n", 
	    OPTS_file_array->chars[j] _ERROR;
	  array_of_seqs->seqs = NULL;
	  array_of_seqs->nb_seqs = 0;
	}
	
	sequence_file_name = OPTS_file_array->chars[j];
	sequencerestart(fd); 
	sequenceparse(); 

	fclose(fd);
	free_flex_mem();
	
	/*If there were some errors print error message, return NULL
	  and free all allocated memory from the parser*/
	if(sequence_nb_errors != 0) {
	  ERROR_ "File '%s' has some errors : process stopped\n\n",
	    OPTS_file_array->chars[j] _ERROR; 
	  array_of_seqs->seqs = NULL;
	  array_of_seqs->nb_seqs = 0;
	  i = 0;
	  while(sequence_result[i] != NULL){
	    if(sequence_result[i]->name != NULL){
	      DESTROY(sequence_result[i]->name);
	    }
	    if(sequence_result[i]->bases != NULL){
	      DESTROY(sequence_result[i]->bases);
	    }
	    DESTROY(sequence_result[i]);
	    i++;
	  }
	  DESTROY(sequence_result);
	}
	else{
	  
	  /*Retrievment of the sequences array*/
	  other_sequences = sequence_result;
	  
	  /*Counting the number of sequences*/
	  while(other_sequences[nb_seqs] != NULL) nb_seqs++;
	  
	  /*If there was not enough sequences return NULL*/
	  if(nb_seqs < 1) {
	    ERROR_ "Not enough sequences....\n" _ERROR;
	    array_of_seqs->seqs = NULL;
	    array_of_seqs->nb_seqs = 0;
	  }
	  else{
	    /*Copy the other sequences in the sequences array*/
	    RENEW(array_of_seqs->seqs,
		  p_sequence,
		  nb_seqs + array_of_seqs->nb_seqs);
	    for(i = 0; i < nb_seqs; i++){
	      NEW(array_of_seqs->seqs[array_of_seqs->nb_seqs],
		  sequence_s,
		  1);
	      NEW(array_of_seqs->seqs[array_of_seqs->nb_seqs]->name,
		  char,
		  strlen(other_sequences[i]->name) + 1);
	      NEW(array_of_seqs->seqs[array_of_seqs->nb_seqs]->bases,
		  char,
		  strlen(other_sequences[i]->bases) + 1);
	      strcpy(array_of_seqs->seqs[array_of_seqs->nb_seqs]->name,
		     other_sequences[i]->name);
	      strcpy(array_of_seqs->seqs[array_of_seqs->nb_seqs]->bases,
		     other_sequences[i]->bases);
	      array_of_seqs->seqs[array_of_seqs->nb_seqs]->length =
		other_sequences[i]->length;
	      array_of_seqs->nb_seqs++;
	    }

	    /*Destroy the temporary sequences array*/
	    for(i = 0; i < nb_seqs; i++){
	      DESTROY(other_sequences[i]->name);
	      DESTROY(other_sequences[i]->bases);
	      DESTROY(other_sequences[i]);
	      
	    }
	    DESTROY(other_sequences);
	    
	  }
	}
      }
	  
      /*Calculation of the real length (all expect N) on all sequences*/
      for(i = 0; i < array_of_seqs->nb_seqs; i++){
	get_real_length_sequence(array_of_seqs->seqs[i]);
	
      }

     } 
    
  } 
}


/////////////////////////////////////////////////////////////////////////////
///
/// \fn void destroy_sequence(p_sequence s)
///
/// \brief Function used to destroy a p_sequence structure
///
/// \param[in,out] s : the p_sequence to destroy
///
///
/// This function will destroy all allocated memory for a p_sequence 
/// structure
///
///
/////////////////////////////////////////////////////////////////////////////

void destroy_sequence(p_sequence s)
{
  
  /*Destroy the sequence if existing*/
  if(s->bases) {
    DESTROY(s->bases);
  }
  
  /*Destroy the name*/
  if(s->name){
    DESTROY(s->name); 
  }

  /*Destroy the species*/
  if(s->species){
    DESTROY(s->species);
  }

  /*Destroy the strain*/
  if(s->strain){
    DESTROY(s->strain);
  }
  
  DESTROY(s);

}


/////////////////////////////////////////////////////////////////////////////
///
/// \fn int is_present_array_sequence(char *seq_name_to_find,
///                                   sequence_array *array_of_seqs)
///
/// \brief Function used to retrieve the index of a sequence according to
/// its name
///
/// \param[in] seq_name_to_find : the name to search for
/// \param[in] array_of_seqs : the array of sequences
///
/// \return the index in the sequence array or -1 if it wasn't found
///
/// This function will search through the array of sequences for the name and 
/// return its idnex or -1 if it wasn't found
///
///
/////////////////////////////////////////////////////////////////////////////


int is_present_array_sequence(char *seq_name_to_find, 
			      sequence_array *array_of_seqs){

  /*Current index*/
  int i;

  /*Search over the array*/
  for(i = 0; i < array_of_seqs->nb_seqs; i++){
    if(strcmp(seq_name_to_find,array_of_seqs->seqs[i]->name) == 0){
      return i;
    }
  }

  return -1;
}


/////////////////////////////////////////////////////////////////////////////
///
/// \fn void get_real_length_sequence(p_sequence seq)
///
/// \brief Function used to calculate the real length of a sequence
///
/// \param[in,out] seq : the reference sequence
///
/// This function will calculate the real length of a sequence(all 
/// nucleotides except N)
///
///
/////////////////////////////////////////////////////////////////////////////


void get_real_length_sequence(p_sequence seq)
{

  /*Index*/
  int i;
  /*Number of N nucleotides*/
  int nb_N = 0;

  /*Read all nucleotides of the sequence and count the number
   of N*/
  for(i = 0; i < seq->length; i++){
    if(seq->bases[i] == 'N'){
      nb_N++;
    }
  }

  seq->real_length = seq->length - nb_N;

}



/////////////////////////////////////////////////////////////////////////////
///
/// \fn char get_complementary_base(char nucleotide)
///
/// \brief Function used to get the complementary nucleotide
///
/// \param[in] nucleotide : the nucleotide to get its complementary
///
/// \return the complementary nucleotide
///
/// This function will return the complementary nucleotide of the one given
/// If bad nucleotide return 'N'
///
///
/////////////////////////////////////////////////////////////////////////////

char get_complementary_base(char nucleotide)
{

  /*Different nucleotide cases*/
  switch(nucleotide){

  case 'A':
    return 'T';
    break;
  case 'T':
    return 'A';
    break;
  case 'C':
    return 'G';
    break;
  case 'G':
    return 'C';
    break;
  case 'R':
    return 'Y';
    break;
  case 'Y':
    return 'R';
    break;
  case 'W':
    return 'W';
    break;
  case 'K':
    return 'M';
    break;
  case 'M':
    return 'K';
    break;
  case 'B':
    return 'V';
    break;
  case 'V':
    return 'B';
    break;
  case 'S':
    return 'S';
    break;
  case 'D':
    return'H';
    break;
  case 'H':
    return 'D';
    break;
  case 'N':
    return 'N';
    break;
    /*Error case : return N*/
  default:
    ERROR_ "Invalid nucleotide %c\n",nucleotide _ERROR;
    return 'N';
  }



}


/////////////////////////////////////////////////////////////////////////////
///
/// \fn void complete_sequences_attributes(sequence_array *array_of_seqs)
///
/// \brief Function used to complete some missing attributes in the sequence
/// array
///
/// \param[in,out] array_of_seqs : the array containing all sequences
/// and that will be updated
///
/// This function will complete the species attribute when possible, put in 
/// uppercase the sequence when needed and replace the spaces in the sequence
/// name by a "_"
///
/////////////////////////////////////////////////////////////////////////////

void complete_sequences_attributes(sequence_array *array_of_seqs)

{

  /*Indexes*/
  int i,j, k;
  /*Number of "|" present in the name : 4 must be present for the name
   to be formatted like :
   gi|49175990|ref|NC_000913.2| Escherichia coli str. K-12 substr. MG1655, complete genome
   so the species can be determined (Escherichia coli) and the strain too
  (K-12 substr. MG1655)*/
  int nb_pipe = 0;
  /*Token used to parse the name*/
  char *token = NULL;
  /*Temporary sequence name*/
  char *temp_name = NULL;
  /*Number of spaces in the name used to retrieve the species and the strain*/
  int nb_spaces = 0;
  /*Length of the species name*/
  int length_species = 0;
  /*Length of the strain name*/
  int length_strain = 0;

  /*Get all sequences*/
  for(i = 0; i < array_of_seqs->nb_seqs; i++){

    /*Test if the first base of the sequence is not in uppercase*/
    if(array_of_seqs->seqs[i]->bases[0] != 
       toupper(array_of_seqs->seqs[i]->bases[0])){
      for(j = 0; j < array_of_seqs->seqs[i]->length; j++){
	array_of_seqs->seqs[i]->bases[j] = 
	  toupper(array_of_seqs->seqs[i]->bases[j]);
      }
    }

    nb_pipe = 0;
    nb_spaces = 0;
    NEW(temp_name, char, strlen(array_of_seqs->seqs[i]->name) + 1);
    strcpy(temp_name,array_of_seqs->seqs[i]->name);

    /*Replace any space by "_" in the sequence name*/
    for(j = 0; j < strlen(array_of_seqs->seqs[i]->name); j++){
      
      if(array_of_seqs->seqs[i]->name[j] == ' '){
	array_of_seqs->seqs[i]->name[j] = '_';
      }
      else{
	if(array_of_seqs->seqs[i]->name[j] == '|'){
	  nb_pipe++;
	}
      }

    }

    

    /*Check species*/
    if(nb_pipe == 4){
      
      length_species = 0;
      length_strain = 0;
      array_of_seqs->seqs[i]->species = NULL;
      array_of_seqs->seqs[i]->strain = NULL;

      /*Split the name according to "|"*/
      token = strtok(temp_name, "|");
      while(token != NULL){
	nb_pipe--;
	/*Species and strain attribute*/
	if(nb_pipe == -1){
	  if(token[0] == ' '){
	    k = 1;
	  }
	  else{
	    k = 0;
	  }
	  while(k != strlen(token)){
	    if(token[k] == ' '){
	      nb_spaces++;
	    }
	    /*Species part*/
	    if(nb_spaces < 2){
	      length_species++;
	      RENEW(array_of_seqs->seqs[i]->species, char, length_species);
	      array_of_seqs->seqs[i]->species[length_species - 1] = token[k];
	    }
	    /*Strain part*/
	    else{
	      /*The strain is all before the coma which is followed by 
	       "complete sequence" or "complete genome". So if a coma
	      is found stop*/
	      if(token[k] == ','){
		RENEW(array_of_seqs->seqs[i]->strain, char, length_strain + 1);
		array_of_seqs->seqs[i]->strain[length_strain] = '\0';
		k = strlen(token) - 1;
	      }
	      else{
		length_strain++;
		RENEW(array_of_seqs->seqs[i]->strain, char, length_strain);
		array_of_seqs->seqs[i]->strain[length_strain - 1] = token[k];
	      }
	      
	    }
	  
	    k++;
	  }
	  RENEW(array_of_seqs->seqs[i]->species, char, length_species + 1);
	  array_of_seqs->seqs[i]->species[length_species] = '\0';

	}
	token = strtok(NULL, "|");
      }
      
    }
    /*No species or strain can be determined*/
    else{
      /*array_of_seqs->seqs[i]->species = NULL;
	array_of_seqs->seqs[i]->strain = NULL;*/
      NEW(array_of_seqs->seqs[i]->species,
	  char,
	  strlen(array_of_seqs->seqs[i]->name) + 1);
      NEW(array_of_seqs->seqs[i]->strain,
	  char,
	  strlen(array_of_seqs->seqs[i]->name) + 1);
      strcpy(array_of_seqs->seqs[i]->species,
	     array_of_seqs->seqs[i]->name);
      strcpy(array_of_seqs->seqs[i]->strain,
	     array_of_seqs->seqs[i]->name);

    }
    DESTROY(temp_name);

  }
}
